@@ -53,6 +53,10 @@ typedef struct _GncCommodityNamespaceClass gnc_commodity_namespaceClass;
 #include <glib/gi18n.h>
 #include "gnc-engine.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* --- type macros --- */
 #define GNC_TYPE_COMMODITY            (gnc_commodity_get_type ())
 #define GNC_COMMODITY(o)              \
@@ -139,13 +143,21 @@ typedef enum
 } QuoteSourceType;
 
 /** This function indicates whether or not the Finance::Quote module
- *  is installed on a users computer.  This includes any other related
+ *  is installed on a user's computer.  This includes any other related
  *  modules that gnucash need to process F::Q information.
  *
  *  @return TRUE is F::Q is installed properly.
  */
 gboolean gnc_quote_source_fq_installed (void);
 
+/** This function returns the version of the Finance::Quote module
+ *  installed on a user's computer. If no proper installation is found
+ *  it will return NULL.
+ *
+ *  @return a version string or NULL
+ */
+const char* gnc_quote_source_fq_version (void);
+
 /** Update gnucash internal tables based on what Finance::Quote
  *  sources are installed.  Sources that have been explicitly coded
  *  into gnucash are marked sensitive/insensitive based upon whether
@@ -155,7 +167,8 @@ gboolean gnc_quote_source_fq_installed (void);
  *  @param sources_list A list of strings containing the source names
  *  as they are known to F::Q.
  */
-void gnc_quote_source_set_fq_installed (const GList *sources_list);
+void gnc_quote_source_set_fq_installed (const char* version_string,
+                                        const GList *sources_list);
 
 /** Return the number of entries for a given type of quote source.
  *
@@ -340,20 +353,6 @@ const char * gnc_commodity_get_mnemonic(const gnc_commodity * cm);
  */
 const char * gnc_commodity_get_namespace(const gnc_commodity * cm);
 
-/** Retrieve the namespace for the specified commodity.  This will be
- *  a pointer to a null terminated string of the form "AMEX",
- *  "NASDAQ", etc.  The only difference between function and
- *  gnc_commodity_get_namespace() is that this function returns
- *  ISO4217 instead of CURRENCY for backward compatibility with the
- *  1.8 data files.
- *
- *  @param cm A pointer to a commodity data structure.
- *
- *  @return A pointer to the namespace for this commodity.  This string
- *  is owned by the engine and should not be freed by the caller.
- */
-const char * gnc_commodity_get_namespace_compat(const gnc_commodity * cm);
-
 /** Retrieve the namespace data strucure for the specified commodity.
  *  This will be a pointer to another data structure.
  *
@@ -1070,6 +1069,10 @@ void gnc_monetary_list_free(MonetaryList *list);
 
 /** @} */
 
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
 #endif /* GNC_COMMODITY_H */
 /** @} */
 /** @} */
