@@ -31,6 +31,10 @@ typedef struct _GncPriceDBClass GNCPriceDBClass;
 #include "gnc-commodity.h"
 #include "gnc-engine.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* --- type macros --- */
 #define GNC_TYPE_PRICE            (gnc_price_get_type ())
 #define GNC_PRICE(o)              \
@@ -116,8 +120,10 @@ GType gnc_pricedb_get_type(void);
       many strings in users' databases, so this string must be
       translated on output instead of always being used in untranslated
       form).
-    - type: the type of quote - types possible right now are bid, ask,
-      last, nav, and unknown.
+    - type: the type of quote - types possible right now are bid, ask, last,
+      nav, transaction, and unknown. 'Transaction' is set when the price is
+      created from an amount and value in a Split and is not available for users
+      to set via the GUI.
 
     \par Implementation Details:
 
@@ -153,9 +159,29 @@ GType gnc_pricedb_get_type(void);
  @{ */
 
 /** */
-typedef struct gnc_price_lookup_s GNCPriceLookup;
 typedef GList PriceList;
 
+/** Price source enum. Be sure to keep in sync with the source_name array in
+ * gnc-pricedb.c. These are in preference order, so for example a quote with
+ * PRICE_SOURCE_EDIT_DLG will overwrite one with PRICE_SOURCE_FQ but not the
+ * other way around.
+ */
+typedef enum
+{
+    PRICE_SOURCE_EDIT_DLG,         // "user:price-editor"
+    PRICE_SOURCE_FQ,               // "Finance::Quote"
+    PRICE_SOURCE_USER_PRICE,       // "user:price"
+    PRICE_SOURCE_XFER_DLG_VAL,     // "user:xfer-dialog"
+    PRICE_SOURCE_SPLIT_REG,        // "user:split-register"
+    PRICE_SOURCE_STOCK_SPLIT,      // "user:stock-split"
+    PRICE_SOURCE_INVOICE,          // "user:invoice-post"
+    PRICE_SOURCE_TEMP,             // "temporary"
+    PRICE_SOURCE_INVALID,          // "invalid"
+} PriceSource;
+
+#define PRICE_TYPE_LAST "last"
+#define PRICE_TYPE_UNK "unknown"
+#define PRICE_TYPE_TRN "transaction"
 /* ------------------ */
 /** @name Constructors
     @{ */
@@ -169,6 +195,17 @@ GNCPrice *gnc_price_create(QofBook *book);
    content-wise duplicate of the given price, p.  The returned clone
    will have a reference count of 1. */
 GNCPrice *gnc_price_clone(GNCPrice* p, QofBook *book);
+
+/** Return a newly-allocated price that's the inverse of the given price, p.
+ *
+ * Inverse means that the commodity and currency are swapped and the value is
+ * the numeric inverse of the original's. The source is set to PRICE_SOURCE_TEMP
+ * to prevent it being saved in the pricedb.
+ * @param p The price to invert
+ * @return a new price, with a ref-count of 1. Don't forget to unref it!
+ */
+GNCPrice *gnc_price_invert(GNCPrice *p);
+
 /**  @} */
 
 /* ------------------ */
@@ -202,7 +239,9 @@ void gnc_price_commit_edit (GNCPrice *p);
 void gnc_price_set_commodity(GNCPrice *p, gnc_commodity *c);
 void gnc_price_set_currency(GNCPrice *p, gnc_commodity *c);
 void gnc_price_set_time(GNCPrice *p, Timespec t);
-void gnc_price_set_source(GNCPrice *p, const char *source);
+void gnc_price_set_time64(GNCPrice *p, time64 t64);
+void gnc_price_set_source(GNCPrice *p, PriceSource source);
+void gnc_price_set_source_string(GNCPrice *p, const char* s);
 void gnc_price_set_typestr(GNCPrice *p, const char* type);
 void gnc_price_set_value(GNCPrice *p, gnc_numeric value);
 /** @} */
@@ -213,13 +252,15 @@ void gnc_price_set_value(GNCPrice *p, gnc_numeric value);
     to the GNCPrice, not copies, so don't free these values.
     @{ */
 
-GNCPrice *      gnc_price_lookup (const GncGUID *guid, QofBook *book);
+    GNCPrice *      gnc_price_lookup (const GncGUID *guid, QofBook *book);
 /*@ dependent @*/
 gnc_commodity * gnc_price_get_commodity(const GNCPrice *p);
 /*@ dependent @*/
 gnc_commodity * gnc_price_get_currency(const GNCPrice *p);
 Timespec        gnc_price_get_time(const GNCPrice *p);
-const char *    gnc_price_get_source(const GNCPrice *p);
+time64          gnc_price_get_time64(const GNCPrice *p);
+PriceSource     gnc_price_get_source(const GNCPrice *p);
+const char *    gnc_price_get_source_string(const GNCPrice *p);
 const char *    gnc_price_get_typestr(const GNCPrice *p);
 gnc_numeric     gnc_price_get_value(const GNCPrice *p);
 gboolean        gnc_price_equal(const GNCPrice *p1, const GNCPrice *p2);
@@ -234,6 +275,15 @@ gboolean        gnc_price_equal(const GNCPrice *p1, const GNCPrice *p2);
 /** This simple function can be useful for debugging the price code */
 void gnc_price_print(GNCPrice *db, FILE *f, int indent);
 /** @} */
+/** @name Denominator Constants Price policy: In order to avoid rounding
+ * problems, currency prices (often called exchange rates) are saved in terms of
+ * the smaller currency, so that price > 1, with a fixed denominator of
+ * 1/1000. Commodity prices in currency are always expressed as value per unit
+ * of the commodity with a fixed denominator of the pricing currency's
+ * SCU * 10000.
+ */
+#define CURRENCY_DENOM 10000
+#define COMMODITY_DENOM_MULT 10000
 
 /* ================================================================ */
 /** @name GNCPrice lists
@@ -280,164 +330,334 @@ gboolean gnc_price_list_equal(PriceList *prices1, PriceList *prices2);
 /** Data type */
 typedef struct gnc_price_db_s GNCPriceDB;
 
-/** return the pricedb associated with the book */
-/*@ dependent @*/
+/** @brief Return the pricedb associated with the book
+ * @param book The QofBook holding the pricedb
+ * @return The GNCPriceDB associated with the book.
+ */
 GNCPriceDB * gnc_pricedb_get_db(QofBook *book);
+/** @brief Return the pricedb via the Book's collection.
+ * @param col The QofCollection holding the pricedb
+ * @return The GNCPriceDB in the QofCollection
+ */
 GNCPriceDB * gnc_collection_get_pricedb(QofCollection *col);
 
-/** gnc_pricedb_destroy - destroy the given pricedb and unref all of
-     the prices it contains.  This may not deallocate all of those
-     prices.  Other code may still be holding references to them. */
+/** @brief Destroy the given pricedb and unref all of the prices it contains.
+ *
+ * This may not deallocate all of those prices.  Other code may still be holding
+ * references to them.
+ * @param db The pricedb to destroy.
+ */
 void gnc_pricedb_destroy(GNCPriceDB *db);
 
-/** Used for editing the pricedb en-mass */
+/** @brief Begin an edit. */
 void gnc_pricedb_begin_edit (GNCPriceDB *);
+/** @brief Commit an edit. */
 void gnc_pricedb_commit_edit (GNCPriceDB *);
 
-/** Indicate whether or not the database is in the middle of a bulk
- *  update.  Setting this flag will disable checks for duplicate
- *  entries. */
+/** @brief Set flag to indicate whether duplication checks should be performed.
+ *
+ * Normally used at load time to speed up loading the pricedb.
+ * @param db The pricedb
+ * @param bulk_update TRUE to disable duplication checks, FALSE to enable them.
+ */
 void gnc_pricedb_set_bulk_update(GNCPriceDB *db, gboolean bulk_update);
 
-/** gnc_pricedb_add_price - add a price to the pricedb, you may drop
-     your reference to the price (i.e. call unref) after this
-     succeeds, whenever you're finished with the price. */
+/** @brief Add a price to the pricedb.
+ *
+ * You may drop your reference to the price (i.e. call unref) after this
+ * succeeds, whenever you're finished with the price.
+ * @param db The pricedb
+ * @param p The GNCPrice to add.
+ * @return TRUE if the price was added, FALSE otherwise.
+ */
 gboolean     gnc_pricedb_add_price(GNCPriceDB *db, GNCPrice *p);
 
-/** gnc_pricedb_remove_price - removes the given price, p, from the
-     pricedb.   Returns TRUE if successful, FALSE otherwise. */
+/** @brief Remove a price from the pricedb and unref the price.
+ * @param db The Pricedb
+ * @param p The price to remove.
+ */
 gboolean     gnc_pricedb_remove_price(GNCPriceDB *db, GNCPrice *p);
 
-gboolean     gnc_pricedb_remove_old_prices(GNCPriceDB *db, Timespec cutoff,
-        const gboolean delete_user, gboolean delete_last);
+typedef enum
+{
+    PRICE_REMOVE_SOURCE_FQ = 1,   // this flag is set when added by F:Q checked
+    PRICE_REMOVE_SOURCE_USER = 2, // this flag is set when added by the user checked
+    PRICE_REMOVE_SOURCE_APP = 4,  // this flag is set when added by the app checked
+    PRICE_REMOVE_SOURCE_COMM = 8, // this flag is set when we have commodities selected
+} PriceRemoveSourceFlags;
 
-/** gnc_pricedb_lookup_latest - find the most recent price for the
-     given commodity in the given currency.  Returns NULL on
-     failure. */
+typedef enum
+{
+    PRICE_REMOVE_KEEP_NONE,           // keep none
+    PRICE_REMOVE_KEEP_LAST_WEEKLY,    // leave last one of every week
+    PRICE_REMOVE_KEEP_LAST_MONTHLY,   // leave last one of every month
+    PRICE_REMOVE_KEEP_LAST_QUARTERLY, // leave last one of every quarter
+    PRICE_REMOVE_KEEP_LAST_PERIOD,    // leave last one of every annual period
+    PRICE_REMOVE_KEEP_SCALED,         // leave one every week then one a month
+} PriceRemoveKeepOptions;
+
+/** @brief Remove and unref prices older than a certain time.
+ * @param db The pricedb
+ * @param comm_list A list of commodities
+ * @param fiscal_end_date the end date of the current accounting period
+ * @param cutoff The time before which prices should be deleted.
+ * @param source Whether Finance::Quote, user or all prices should be deleted.
+ * @param keep Whether scaled, monthly, weekly or no prices should be left.
+ * @return True if there were prices to process, False if not.
+ */
+gboolean     gnc_pricedb_remove_old_prices(GNCPriceDB *db, GList *comm_list,
+                                           GDate *fiscal_end_date, Timespec cutoff,
+                                           PriceRemoveSourceFlags source,
+                                           PriceRemoveKeepOptions keep);
+
+/** @brief Find the most recent price between the two commodities.
+ *
+ * The returned GNCPrice may be in either direction so check to ensure that its
+ * value is correctly applied.
+ * @param db The pricedb
+ * @param commodity The first commodity
+ * @param currency The second commodity
+ * @return A GNCPrice or NULL if no price exists.
+ */
 GNCPrice   * gnc_pricedb_lookup_latest(GNCPriceDB *db,
                                        const gnc_commodity *commodity,
                                        const gnc_commodity *currency);
 
-/** gnc_pricedb_lookup_latest_any_currency - find the most recent prices
-     for the given commodity in any available currency. Prices will be
-     returned as a GNCPrice list (see above). */
+/** @brief Find the most recent price between a commodity and all other
+ * commodities
+ *
+ * The returned GNCPrices may be in either direction so check to ensure that
+ * their values are correctly applied.
+ * @param db The pricedb
+ * @param commodity The commodity for which to obtain prices
+ * @return A PriceList of prices found, or NULL if none found.
+ */
 PriceList * gnc_pricedb_lookup_latest_any_currency(GNCPriceDB *db,
         const gnc_commodity *commodity);
 
-/** gnc_pricedb_has_prices - return an indication of whether or not
-    there are any prices for a given commodity in the given currency.
-    Returns TRUE if there are prices, FALSE otherwise. */
+/** @brief Report whether the pricedb contains prices for one commodity in
+ * another.
+ *
+ * Does *not* check the reverse direction.
+ * @param db The pricedb to check
+ * @param commodity The commodity to check for the existence of prices
+ * @param currency The commodity in which prices are sought. If NULL reports all
+ * commodities.
+ * @return TRUE if matching prices are found, FALSE otherwise.
+ */
 gboolean     gnc_pricedb_has_prices(GNCPriceDB *db,
                                     const gnc_commodity *commodity,
                                     const gnc_commodity *currency);
 
-/** gnc_pricedb_get_prices - return all the prices for a given
-     commodity in the given currency.  Returns NULL on failure.  The
-     result is a GNCPrice list (see above).  */
+/** @brief Return all the prices for a given commodity in another.
+ *
+ * Does *not* retrieve reverse prices, i.e. prices of the second commodity in
+ * the first.
+ * @param db The pricedb from which to retrieve prices.
+ * @param commodity The commodity for which prices should be retrieved.
+ * @param currency The commodity in which prices should be quoted. If NULL, all
+ * prices in any commodity are included.
+ * @return A PriceList of matching prices or NULL if none were found.
+*/
 PriceList * gnc_pricedb_get_prices(GNCPriceDB *db,
                                    const gnc_commodity *commodity,
                                    const gnc_commodity *currency);
 
-/** gnc_pricedb_lookup_at_time - return all prices that match the given
-     commodity, currency, and timespec.  Prices will be returned as a
-     GNCPrice list (see above). */
-PriceList * gnc_pricedb_lookup_at_time(GNCPriceDB *db,
+/** @brief Find the price between two commodities at a timespec.
+ *
+ * The returned GNCPrice may be in either direction so check to ensure that its
+ * value is correctly applied.
+ * @param db The pricedb
+ * @param commodity The first commodity
+ * @param currency The second commodity
+ * @param t The timespec at which to retrieve the price.
+ * @return A GNCPrice or NULL if none matches.
+ */
+/* NOT USED */
+GNCPrice * gnc_pricedb_lookup_at_time(GNCPriceDB *db,
                                        const gnc_commodity *commodity,
                                        const gnc_commodity *currency,
                                        Timespec t);
 
-/** gnc_pricedb_lookup_at_time_any_currency - return all prices that match the
-     given commodity and timespec in any available currency.  Prices will be
-     returned as a GNCPrice list (see above). */
-PriceList * gnc_pricedb_lookup_at_time_any_currency(GNCPriceDB *db,
-        const gnc_commodity *c,
-        Timespec t);
+/** @brief Return the price between the two commodities on the indicated
+ * day. Note that the notion of day might be distorted by changes in timezone.
+ *
+ * The returned GNCPrice may be in either direction so check to ensure that its
+ * value is correctly applied.
+ * @param db The pricedb
+ * @param commodity The first commodity
+ * @param currency The second commodity
+ * @param t A time. The price returned will be in the same day as this time
+ * according to the local timezone.
+ * @return A GNCPrice or NULL on failure.
+ */
+GNCPrice * gnc_pricedb_lookup_day(GNCPriceDB *db,
+                                  const gnc_commodity *commodity,
+                                  const gnc_commodity *currency,
+                                  Timespec t);
 
-/** gnc_pricedb_lookup_day - return all prices that match the given
-     commodity, currency, and timespec.  Prices will be returned as a
-     GNCPrice list (see above). */
-PriceList * gnc_pricedb_lookup_day(GNCPriceDB *db,
-                                   const gnc_commodity *commodity,
-                                   const gnc_commodity *currency,
-                                   Timespec t);
+GNCPrice * gnc_pricedb_lookup_day_t64(GNCPriceDB *db,
+                                  const gnc_commodity *commodity,
+                                  const gnc_commodity *currency,
+                                  time64 t64);
 
-/** gnc_pricedb_lookup_day_any_currency - return all prices that match the
-     given commodity and timespec in any available currency.  Prices will be
-     returned as a GNCPrice list (see above). */
-PriceList * gnc_pricedb_lookup_day_any_currency(GNCPriceDB *db,
-        const gnc_commodity *c,
-        Timespec t);
-
-/** gnc_pricedb_lookup_nearest_in_time - return the price for the given
-     commodity in the given currency nearest to the given time t. */
+/** @brief Return the price between the two commoditiesz nearest to the given
+ * time.
+ *
+ * The returned GNCPrice may be in either direction so check to ensure that its
+ * value is correctly applied.
+ * @param db The pricedb
+ * @param c The first commodity
+ * @param currency The second commodity
+ * @param t The time nearest to which the returned price should be.
+ * @return A GNCPrice or NULL if no prices exist between the two commodities.
+ */
 GNCPrice   * gnc_pricedb_lookup_nearest_in_time(GNCPriceDB *db,
         const gnc_commodity *c,
         const gnc_commodity *currency,
         Timespec t);
 
-/** gnc_pricedb_lookup_nearest_in_time_any_currency - return all prices that
-     match the given commodity and timespec in any available currency. Prices
-     will be returned as a GNCPrice list (see above). */
+GNCPrice   * gnc_pricedb_lookup_nearest_in_time64(GNCPriceDB *db,
+        const gnc_commodity *c,
+        const gnc_commodity *currency,
+        time64 t);
+
+/** @brief Return the price nearest in time to that given between the given
+ * commodity and every other.
+ *
+ * The returned GNCPrices may be in either direction so check to ensure that
+ * their values are correctly applied.
+ *
+ * @param db, The pricedb
+ * @param c, The commodity for which prices should be obtained.
+ * @param t, The time nearest to which the prices should be obtained.
+ * @return A PriceList of prices for each commodity pair found or NULL if none
+ * are.
+ */
 PriceList * gnc_pricedb_lookup_nearest_in_time_any_currency(GNCPriceDB *db,
         const gnc_commodity *c,
         Timespec t);
-/** gnc_pricedb_lookup_latest_before - return the latest price for the given commodity
-    in the given currency up to and including time t. */
+PriceList * gnc_pricedb_lookup_nearest_in_time_any_currency_t64(GNCPriceDB *db,
+        const gnc_commodity *c,
+        time64 t64);
+
+/** @brief Return the latest price between the given commodities before the
+ * given time.
+ *
+ * The returned GNCPrice may be in either direction so check to ensure that its
+ * value is correctly applied.
+ * @param db The pricedb
+ * @param c The first commodity
+ * @param currency The second commodity
+ * @param t The time before which to find the price
+ * @return A GNCPrice or NULL if no prices are found before t.
+ */
+/* NOT USED, but see bug 743753 */
 GNCPrice * gnc_pricedb_lookup_latest_before(GNCPriceDB *db,
         gnc_commodity *c,
         gnc_commodity *currency,
         Timespec t);
 
-/** gnc_pricedb_lookup_latest_before_any_currency - return recent prices that
-     match the given commodity up to and including time t in any available currency. Prices
-     will be returned as a GNCPrice list (see above). */
+/** @brief Return the latest price between the given commodity and any other
+ * before the given time.
+ *
+ * The returned GNCPrice may be in either direction so check to ensure that its
+ * value is correctly applied.
+ * @param db The pricedb
+ * @param c The commodity
+ * @param t The time before which to find prices
+ * @return A PriceList of prices for each commodity found or NULL if none are.
+ */
+/* NOT USED, but see bug 743753 */
 PriceList * gnc_pricedb_lookup_latest_before_any_currency(GNCPriceDB *db,
-        gnc_commodity *c,
-        Timespec t);
+                                                         const gnc_commodity *c,
+                                                          Timespec t);
 
 
-/** gnc_pricedb_convert_balance_latest_price - Convert a balance
-    from one currency to another. */
+/** @brief Convert a balance from one currency to another using the most recent
+ * price between the two.
+ * @param pdb The pricedb
+ * @param balance The balance to be converted
+ * @param balance_currency The commodity in which the balance is currently
+ * expressed
+ * @param new_currency The commodity to which the balance should be converted
+ * @return A new balance or gnc_numeric_zero if no price is available.
+ */
 gnc_numeric
 gnc_pricedb_convert_balance_latest_price(GNCPriceDB *pdb,
-        gnc_numeric balance,
-        const gnc_commodity *balance_currency,
-        const gnc_commodity *new_currency);
+                                         gnc_numeric balance,
+                                         const gnc_commodity *balance_currency,
+                                         const gnc_commodity *new_currency);
 
-/** gnc_pricedb_convert_balance_nearest_price - Convert a balance
-    from one currency to another. */
+/** @brief Convert a balance from one currency to another using the price
+ * nearest to the given time.
+ * @param pdb The pricedb
+ * @param balance The balance to be converted
+ * @param balance_currency The commodity in which the balance is currently
+ * expressed
+ * @param new_currency The commodity to which the balance should be converted
+ * @param t The time nearest to which price should be used.
+ * @return A new balance or gnc_numeric_zero if no price is available.
+ */
 gnc_numeric
 gnc_pricedb_convert_balance_nearest_price(GNCPriceDB *pdb,
-        gnc_numeric balance,
-        const gnc_commodity *balance_currency,
-        const gnc_commodity *new_currency,
-        Timespec t);
+                                          gnc_numeric balance,
+                                          const gnc_commodity *balance_currency,
+                                          const gnc_commodity *new_currency,
+                                          time64 t);
 
-/** gnc_pricedb_convert_balance_latest_before - Convert a balance from one currency
-    to another using the lastest price prior to Timespec t. */
-gnc_numeric
-gnc_pricedb_convert_balance_latest_before(GNCPriceDB *pdb,
-        gnc_numeric balance,
-        gnc_commodity *balance_currency,
-        gnc_commodity *new_currency,
-        Timespec t);
+typedef gboolean (*GncPriceForeachFunc)(GNCPrice *p, gpointer user_data);
 
-
-/** gnc_pricedb_foreach_price - call f once for each price in db, until
-     and unless f returns FALSE.  If stable_order is not FALSE, make
-     sure the ordering of the traversal is stable (i.e. the same order
-     every time given the same db contents -- stable traversals may be
-     less efficient).  */
+/** @brief Call a GncPriceForeachFunction once for each price in db, until the
+ * function returns FALSE.
+ *
+ * If stable_order is not FALSE, make sure the ordering of the traversal is
+ * stable (i.e. the same order every time given the same db contents -- stable
+ * traversals may be less efficient).
+ * @param db The pricedb
+ * @param f The function to call
+ * @param user_data A data to pass to each invocation of f
+ * @param stable_order Ensure that the traversal is performed in the same order
+ * each time.
+ * @return TRUE if all calls to f succeeded (unstable) or if the order of
+ * processing was the same as the previous invocation (stable), FALSE otherwise.
+ */
 gboolean     gnc_pricedb_foreach_price(GNCPriceDB *db,
-                                       gboolean (*f)(GNCPrice *p,
-                                               gpointer user_data),
+                                       GncPriceForeachFunc f,
                                        gpointer user_data,
                                        gboolean stable_order);
 
-/** gnc_pricedb_get_num_prices - return the number of prices
-   in the database. */
+/** @brief Get the number of prices, in any currency, for a given commodity.
+ * @param db The pricedb
+ * @param c The commodity
+ * @return The number of prices in the database for this commody, zero if none
+ */
+int
+gnc_pricedb_num_prices(GNCPriceDB *db,
+                       const gnc_commodity *c);
+
+/** @brief Get the nth price for the given commodity in  reverse date order
+ * @param db The pricedb
+ * @param c The commodity whose nth price is needed
+ * @param n Zero based index of the price wanted
+ * @return The nth price for this commodity in reverse chronological order, without
+ * regard for what currency the price is in
+ */
+GNCPrice *
+gnc_pricedb_nth_price (GNCPriceDB *db,
+                       const gnc_commodity *c,
+                       const int n);
+
+/* The following two convenience functions are used to test the xml backend */
+/** @brief Return the number of prices in the database.
+ *
+ * For XML Backend Testing
+ */
 guint gnc_pricedb_get_num_prices(GNCPriceDB *db);
 
+/** @brief Test equality of two pricedbs
+ *
+ * For XML Backend Testing */
 gboolean gnc_pricedb_equal (GNCPriceDB *db1, GNCPriceDB *db2);
 
 /** @name Internal/Debugging
@@ -460,6 +680,10 @@ void gnc_pricedb_print_contents(GNCPriceDB *db, FILE *f);
 
 /** @} */
 
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
 #endif /* GNC_PRICEDB_H */
 /** @} */
 /** @} */
