@@ -84,6 +84,11 @@
 #ifndef _QOF_LOG_H
 #define _QOF_LOG_H
 
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <glib.h>
@@ -101,8 +106,8 @@
 
 DEFINE_ENUM (QofLogLevel, LOG_LEVEL_LIST)
 
-gchar* qof_log_level_to_string(QofLogLevel lvl);
-QofLogLevel qof_log_level_from_string(const gchar *str);
+const char* qof_log_level_to_string(QofLogLevel lvl);
+QofLogLevel qof_log_level_from_string(const char *str);
 
 /** Indents one level; see ENTER macro. **/
 void qof_log_indent(void);
@@ -169,7 +174,13 @@ void qof_log_set_default(QofLogLevel log_level);
 #define PRETTY_FUNC_NAME qof_log_prettify(G_STRFUNC)
 
 #ifdef _MSC_VER
-/* Microsoft Visual Studio */
+/* Microsoft Visual Studio: MSVC compiler has a different syntax for
+ * macros with variadic argument list. */
+
+/* TODO: After the C++2a feature __VA_OPT__ gets implemented in both
+ * flavors, it should be inserted before __VA_ARGS__ and the else branch
+ * gets obsolete and should be removed.
+ */
 
 /** Log a fatal error */
 #define FATAL(format, ...) do { \
@@ -288,6 +299,10 @@ void qof_log_set_default(QofLogLevel log_level);
 } while (0);
 
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* _QOF_LOG_H */
 
 /** @} */
