@@ -27,9 +27,8 @@
 (define-module (gnucash report job-report))
 
 (use-modules (srfi srfi-1))
-(use-modules (gnucash printf))
 (use-modules (gnucash gnc-module))
-(use-modules (gnucash main))		; for gnc:debug
+(use-modules (gnucash utilities))		; for gnc:debug
 (use-modules (gnucash gettext))
 
 (gnc:module-load "gnucash/report/report-system" 0)
@@ -47,16 +46,6 @@
 (define desc-header (N_ "Description"))
 (define amount-header (N_ "Amount"))
 
-(define-macro (addto! alist element)
-  `(set! ,alist (cons ,element ,alist)))
-
-(define (set-last-row-style! table tag . rest)
-  (let ((arg-list 
-         (cons table 
-               (cons (- (gnc:html-table-num-rows table) 1)
-                     (cons tag rest)))))
-    (apply gnc:html-table-set-row-style! arg-list)))
-
 (define (date-col columns-used)
   (vector-ref columns-used 0))
 (define (date-due-col columns-used)
@@ -132,7 +121,7 @@
 	(table (gnc:make-html-table)))
 
     (define (in-interval this-date current-bucket)
-      (gnc:timepair-lt this-date current-bucket))
+      (< this-date current-bucket))
 
     (define (find-bucket current-bucket bucket-intervals date)
       (begin
@@ -189,13 +178,12 @@
 (define (make-row column-vector date due-date num type-str memo monetary)
   (let ((row-contents '()))
     (if (date-col column-vector)
-	(addto! row-contents (gnc-print-date date)))
+	(addto! row-contents (qof-print-date date)))
     (if (date-due-col column-vector)
 	(addto! row-contents 
-		(if (and due-date
-			 (not (equal? due-date (cons 0 0))))
-		    (gnc-print-date due-date)
-		    "")))
+                (if due-date
+                    (qof-print-date due-date)
+                    "")))
     (if (num-col column-vector)
 	(addto! row-contents num))
     (if (type-col column-vector)
@@ -236,7 +224,7 @@
 (define (add-txn-row table txn acc column-vector odd-row? printed?
 		     inv-str reverse? start-date total)
   (let* ((type (xaccTransGetTxnType txn))
-	 (date (gnc-transaction-get-date-posted txn))
+	 (date (xaccTransGetDate txn))
 	 (due-date #f)
 	 (value (xaccTransGetAccountValue txn acc))
 	 (split (xaccTransGetSplit txn 0))
@@ -258,15 +246,16 @@
     (if reverse?
 	(set! value (gnc-numeric-neg value)))
 
-    (if (gnc:timepair-later start-date date)
+    (if (< start-date date)
 	(begin
 	  
 	  ; Adds 'balance' row if needed
 	  (set! printed? (add-balance-row table column-vector txn odd-row? printed? start-date total))
 	  
 	  ; Now print out the invoice row
-	  (if (not (null? invoice))
-	      (set! due-date (gncInvoiceGetDateDue invoice)))
+          (if (and (not (null? invoice))
+                   (gncInvoiceIsPosted invoice))
+              (set! due-date (gncInvoiceGetDateDue invoice)))
 
 	  (let ((row (make-row column-vector date due-date (gnc-get-num-action txn split)
 			       type-str (xaccSplitGetMemo split)
@@ -406,13 +395,7 @@
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
     (N_ "Display Columns") amount-header
-    "hb" (N_ "Display the transaction amount?") #t)) 
-
-  (gnc:register-inv-option
-   (gnc:make-string-option
-    gnc:pagename-general (N_ "Today Date Format")
-    "p" (N_ "The format for the date->string conversion for today's date.")
-    (gnc-default-strftime-date-format)))
+    "hb" (N_ "Display the transaction amount?") #t))
 
   (gnc:options-set-default-section gnc:*report-options* "General")
 
@@ -471,7 +454,7 @@
      guid QOF-QUERY-OR)
 
     (xaccQueryAddSingleAccountMatch q account QOF-QUERY-AND)
-    (xaccQueryAddDateMatchTS q #f end-date #t end-date QOF-QUERY-AND)
+    (xaccQueryAddDateMatchTT q #f end-date #t end-date QOF-QUERY-AND)
     (qof-query-set-book q (gnc-get-current-book))
     q))
 
@@ -485,11 +468,11 @@
     (gnc:html-table-append-row!
      table
      (list
-      (string-expand (gnc:owner-get-name-and-address-dep owner) #\newline "<br>")))
+      (string-expand (gnc:owner-get-name-and-address-dep owner) #\newline "<br/>")))
     (gnc:html-table-append-row!
      table
-     (list "<br>"))
-    (set-last-row-style!
+     (list "<br/>"))
+    (gnc:html-table-set-last-row-style!
      table "td"
      'attribute (list "valign" "top"))
     table))
@@ -499,7 +482,7 @@
    table
    (list
     (string-append label ":&nbsp;")
-    (string-expand (gnc-print-date date) #\space "&nbsp;"))))
+    (string-expand (qof-print-date date) #\space "&nbsp;"))))
 
 (define (make-date-table)
   (let ((table (gnc:make-html-table)))
@@ -507,20 +490,15 @@
      table "table"
      'attribute (list "border" 0)
      'attribute (list "cellpadding" 0))
-    (set-last-row-style!
+    (gnc:html-table-set-last-row-style!
      table "td"
      'attribute (list "valign" "top"))
     table))
 
 (define (make-myname-table book date-format)
   (let* ((table (gnc:make-html-table))
-	 (slots (qof-book-get-slots book))
-	 (name (kvp-frame-get-slot-path-gslist
-		slots (append gnc:*kvp-option-path*
-			      (list gnc:*business-label* gnc:*company-name*))))
-	 (addy (kvp-frame-get-slot-path-gslist
-		slots (append gnc:*kvp-option-path*
-			      (list gnc:*business-label* gnc:*company-addy*)))))
+	 (name (gnc:company-info book gnc:*company-name*))
+	 (addy (gnc:company-info book gnc:*company-addy*)))
 
     (gnc:html-table-set-style!
      table "table"
@@ -533,11 +511,11 @@
     (gnc:html-table-append-row! table (list (if name name "")))
     (gnc:html-table-append-row! table (list (string-expand
 					     (if addy addy "")
-					     #\newline "<br>")))
+					     #\newline "<br/>")))
     (gnc:html-table-append-row! table (list
 				       (strftime
 					date-format
-					(gnc-localtime (car (gnc:get-today))))))
+					(gnc-localtime (current-time)))))
     table))
 
 (define (make-break! document)
@@ -557,17 +535,17 @@
 	 (query (qof-query-create-for-splits))
 	 (account (opt-val owner-page acct-string))
 	 (owner (opt-val owner-page owner-string))
-	 (start-date (gnc:timepair-start-day-time 
+	 (start-date (gnc:time64-start-day-time 
 		       (gnc:date-option-absolute-time
 			(opt-val gnc:pagename-general (N_ "From")))))
-	 (end-date (gnc:timepair-end-day-time 
+	 (end-date (gnc:time64-end-day-time 
 		       (gnc:date-option-absolute-time
 			(opt-val gnc:pagename-general (N_ "To")))))
-	 (book (gnc-get-current-book)) ;XXX Grab this from elsewhere
+	 (book (gnc-account-get-book account))
+         (date-format (gnc:options-fancy-date book))
 	 (type (opt-val "__reg" "owner-type"))
 	 (type-str "")
          (report-title-str ""))
-
     (cond
       ((eqv? type GNC-OWNER-CUSTOMER)
        (set! type-str (N_ "Customer"))
@@ -617,7 +595,7 @@
 
 	  (gnc:html-document-add-object!
 	   document
-	   (make-myname-table book (opt-val gnc:pagename-general (N_ "Today Date Format"))))
+	   (make-myname-table book date-format))
 
 	  (gnc:html-document-add-object!
 	   document
@@ -631,9 +609,9 @@
 	    (string-append
 	     (_ "Date Range")
 	     ": "
-	     (gnc-print-date start-date)
+	     (qof-print-date start-date)
 	     " - "
-	     (gnc-print-date end-date))))
+	     (qof-print-date end-date))))
 
 	  (make-break! document)
 
@@ -643,9 +621,9 @@
 	(gnc:html-document-add-object!
 	 document
 	 (gnc:make-html-text
-	  (sprintf #f 
-		   (_ "No valid %s selected. Click on the Options button to select a company.")
-		   (_ type-str))))) ;; FIXME because of translations: Please change this string into full sentences instead of sprintf, because in non-english languages the "no valid" has different forms depending on the grammatical gender of the "%s".
+	  (format #f 
+		   (_ "No valid ~a selected. Click on the Options button to select a company.")
+		   (_ type-str))))) ;; FIXME because of translations: Please change this string into full sentences instead of format, because in non-english languages the "no valid" has different forms depending on the grammatical gender of the "%s".
 
     (qof-query-destroy query)
     document))
