@@ -5,6 +5,7 @@
 ;;
 ;; Created by:  Derek Atkins <warlord@MIT.EDU>
 ;; Copyright (c) 2002, 2003 Derek Atkins <warlord@MIT.EDU>
+;; Modified by AMM to show tax figures of invoice.
 ;;
 ;; This program is free software; you can redistribute it and/or    
 ;; modify it under the terms of the GNU General Public License as   
@@ -28,7 +29,7 @@
 
 (use-modules (srfi srfi-1))
 (use-modules (gnucash gnc-module))
-(use-modules (gnucash main))        ; for gnc:debug
+(use-modules (gnucash utilities))        ; for gnc:debug
 (use-modules (gnucash gettext))
 
 (gnc:module-load "gnucash/report/report-system" 0)
@@ -52,6 +53,8 @@
 (define reference-header (N_ "Reference"))
 (define type-header (N_ "Type"))
 (define desc-header (N_ "Description"))
+(define sale-header (N_ "Sale"))
+(define tax-header (N_ "Tax"))
 (define credit-header (N_ "Credits"))
 (define debit-header (N_ "Debits"))
 (define amount-header (N_ "Amount"))
@@ -70,20 +73,23 @@
   (cond ((eqv? owner-type GNC-OWNER-CUSTOMER) (N_ "Customer"))
         ((eqv? owner-type GNC-OWNER-EMPLOYEE) (N_ "Employee"))
         ;; FALL THROUGH
-        (else (N_ "Company")))) 
+        (else
+          (N_ "Company")))) 
 
 ;; Error strings in case there is no (valid) selection (translated)
 (define (invalid-selection-title-string owner-type)
   (cond ((eqv? owner-type GNC-OWNER-CUSTOMER) (_ "No valid customer selected."))
         ((eqv? owner-type GNC-OWNER-EMPLOYEE) (_ "No valid employee selected."))
         ;; FALL THROUGH
-        (else (_ "No valid company selected."))))
+        (else
+          (_ "No valid company selected."))))
 
 (define (invalid-selection-string owner-type)
   (cond ((eqv? owner-type GNC-OWNER-CUSTOMER) (_ "This report requires a customer to be selected."))
         ((eqv? owner-type GNC-OWNER-EMPLOYEE) (_ "This report requires a employee to be selected."))
         ;; FALL THROUGH
-        (else (_ "This report requires a company to be selected."))))
+        (else
+          (_ "This report requires a company to be selected."))))
 
 ;; Html formatted error message documents
 (define (gnc:html-make-no-owner-warning
@@ -108,17 +114,8 @@
   (cond ((eqv? owner-type GNC-OWNER-CUSTOMER) (_ "Customer"))
         ((eqv? owner-type GNC-OWNER-EMPLOYEE) (_ "Employee"))
         ;; FALL THROUGH
-        (else (_ "Vendor")))) 
-
-(define-macro (addto! alist element)
-  `(set! ,alist (cons ,element ,alist)))
-
-(define (set-last-row-style! table tag . rest)
-  (let ((arg-list 
-         (cons table 
-               (cons (- (gnc:html-table-num-rows table) 1)
-                     (cons tag rest)))))
-    (apply gnc:html-table-set-row-style! arg-list)))
+        (else
+          (_ "Vendor")))) 
 
 (define (date-col columns-used)
   (vector-ref columns-used 0))
@@ -130,14 +127,18 @@
   (vector-ref columns-used 3))
 (define (memo-col columns-used)
   (vector-ref columns-used 4))
-(define (credit-col columns-used)
+(define (sale-col columns-used)
   (vector-ref columns-used 5))
-(define (debit-col columns-used)
+(define (tax-col columns-used)
   (vector-ref columns-used 6))
-(define (value-col columns-used)
+(define (credit-col columns-used)
   (vector-ref columns-used 7))
+(define (debit-col columns-used)
+  (vector-ref columns-used 8))
+(define (value-col columns-used)
+  (vector-ref columns-used 9))
 
-(define columns-used-size 8)
+(define columns-used-size 10)
 
 (define (build-column-used options)   
   (define (opt-val section name)
@@ -159,9 +160,11 @@
     (set-col (opt-val "Display Columns" reference-header) 2)
     (set-col (opt-val "Display Columns" type-header) 3)
     (set-col (opt-val "Display Columns" desc-header) 4)
-    (set-col (opt-val "Display Columns" credit-header) 5)
-    (set-col (opt-val "Display Columns" debit-header) 6)
-    (set-col (opt-val "Display Columns" amount-header) 7)
+    (set-col (opt-val "Display Columns" sale-header) 5)
+    (set-col (opt-val "Display Columns" tax-header) 6)
+    (set-col (opt-val "Display Columns" credit-header) 7)
+    (set-col (opt-val "Display Columns" debit-header) 8)
+    (set-col (opt-val "Display Columns" amount-header) 9)
     col-vector))
 
 (define (make-heading-list column-vector)
@@ -176,6 +179,10 @@
     (addto! heading-list (_ type-header)))
     (if (memo-col column-vector)
     (addto! heading-list (_ desc-header)))
+    (if (sale-col column-vector)
+    (addto! heading-list (_ sale-header)))
+    (if (tax-col column-vector)
+    (addto! heading-list (_ tax-header)))
     (if (credit-col column-vector)
     (addto! heading-list (_ credit-header)))
     (if (debit-col column-vector)
@@ -202,15 +209,14 @@
     (define oldintervalreversed (reverse (make-interval-list to-date)))          
     (reverse (cons dayforcurrent oldintervalreversed))) 
 
-(define (make-aging-table options query bucket-intervals reverse? date-type)
+(define (make-aging-table options query bucket-intervals reverse? date-type currency)
   (let ((lots (xaccQueryGetLots query QUERY-TXN-MATCH-ANY))
     (buckets (new-bucket-vector))
     (payments (gnc-numeric-zero))
-    (currency (gnc-default-currency)) ;XXX
     (table (gnc:make-html-table)))
 
      (define (in-interval this-date current-bucket)
-      (gnc:timepair-lt this-date current-bucket))
+      (< this-date current-bucket))
 
      (define (find-bucket current-bucket bucket-intervals date)
       (begin
@@ -235,7 +241,7 @@
        (let* ((bal (gnc-lot-get-balance lot))
           (invoice (gncInvoiceGetInvoiceFromLot lot))
               (date (if (eq? date-type 'postdate)
-               (gncInvoiceGetDatePosted invoice) 
+               (gncInvoiceGetDatePostedTT invoice) 
                (gncInvoiceGetDateDue invoice)))
               )
          
@@ -268,15 +274,14 @@
 ;;
 ;; Make a row list based on the visible columns
 ;;
-(define (make-row column-vector date due-date num type-str memo monetary credit debit)
+(define (make-row column-vector date due-date num type-str memo monetary credit debit sale tax)
   (let ((row-contents '()))
     (if (date-col column-vector)
-        (addto! row-contents (gnc-print-date date)))
+        (addto! row-contents (qof-print-date date)))
     (if (date-due-col column-vector)
         (addto! row-contents 
-         (if (and due-date
-              (not (equal? due-date (cons 0 0))))
-             (gnc-print-date due-date)
+         (if due-date
+             (qof-print-date due-date)
              "")))
     (if (num-col column-vector)
         (addto! row-contents num))
@@ -284,6 +289,12 @@
         (addto! row-contents type-str))
     (if (memo-col column-vector)
         (addto! row-contents memo))
+    (if (sale-col column-vector)
+        (addto! row-contents
+         (gnc:make-html-table-cell/markup "number-cell" sale)))
+    (if (tax-col column-vector)
+        (addto! row-contents
+         (gnc:make-html-table-cell/markup "number-cell" tax)))
     (if (credit-col column-vector)
         (addto! row-contents
          (gnc:make-html-table-cell/markup "number-cell" credit)))
@@ -307,7 +318,7 @@
     (set! printed? #t)
     (if (and (value-col column-vector) (not (gnc-numeric-zero-p total)))
         (let ((row (make-row column-vector start-date #f "" (_ "Balance") ""
-                 (gnc:make-gnc-monetary (xaccTransGetCurrency txn) total) "" ""))
+                 (gnc:make-gnc-monetary (xaccTransGetCurrency txn) total) "" "" "" ""))
           (row-style (if odd-row? "normal-row" "alternate-row")))
           (gnc:html-table-append-row/markup! table row-style (reverse row))
           (set! odd-row? (not odd-row?))
@@ -324,9 +335,11 @@
 (define (add-txn-row table txn acc column-vector odd-row? printed?
              reverse? start-date total)
   (let* ((type (xaccTransGetTxnType txn))
-     (date (gnc-transaction-get-date-posted txn))
+     (date (xaccTransGetDate txn))
      (due-date #f)
      (value (xaccTransGetAccountValue txn acc))
+     (sale (gnc-numeric-zero))
+     (tax (gnc-numeric-zero))
      (split (xaccTransGetSplit txn 0))
      (invoice (gncInvoiceGetInvoiceFromTxn txn))
      (currency (xaccTransGetCurrency txn))
@@ -349,7 +362,7 @@
    (if reverse?
     (set! value (gnc-numeric-neg value)))
 
-   (if (gnc:timepair-le start-date date)
+   (if (<= start-date date)
     (begin
       
       ; Adds 'balance' row if needed
@@ -357,7 +370,16 @@
       
       ; Now print out the invoice row
       (if (not (null? invoice))
-          (set! due-date (gncInvoiceGetDateDue invoice)))
+        (begin
+          (set! due-date (and (gncInvoiceIsPosted invoice)
+                              (gncInvoiceGetDateDue invoice)))
+          (set! sale (gncInvoiceGetTotalSubtotal invoice))
+          (set! tax (gncInvoiceGetTotalTax invoice))))
+
+      (if (gncInvoiceGetIsCreditNote invoice)
+        (begin
+          (set! tax (gnc-numeric-neg tax))
+          (set! sale (gnc-numeric-neg sale))))
 
       (let ((row (make-row column-vector date due-date (gnc-get-num-action txn split)
                    type-str (xaccSplitGetMemo split)
@@ -366,6 +388,10 @@
                (gnc:make-gnc-monetary currency value) "")
            (if (gnc-numeric-negative-p value)
                (gnc:make-gnc-monetary currency value) "")
+           (if (not (null? invoice))
+               (gnc:make-gnc-monetary currency sale) "")
+           (if (not (null? invoice))
+               (gnc:make-gnc-monetary currency tax) "")
         ))
         (row-style (if odd-row? "normal-row" "alternate-row")))
 
@@ -375,7 +401,7 @@
       (set! odd-row? (not odd-row?))
       ))
 
-    (list printed? value odd-row?)
+    (list printed? value odd-row? sale tax)
     ))
 
 
@@ -385,7 +411,10 @@
     (total (gnc-numeric-zero))
     (debit (gnc-numeric-zero))
     (credit (gnc-numeric-zero))
-    (currency (gnc-default-currency)) ;XXX
+
+    (tax (gnc-numeric-zero))
+    (sale (gnc-numeric-zero))
+    (currency (xaccAccountGetCommodity acc))
     (table (gnc:make-html-table))
     (reverse? (gnc:option-value (gnc:lookup-option options "__reg"
                               "reverse?"))))
@@ -410,9 +439,12 @@
 
           (set! printed? (car result))
           (if (and printed? total)
-        (if (gnc-numeric-negative-p (cadr result))
-            (set! debit (gnc-numeric-add-fixed debit (cadr result)))
-            (set! credit (gnc-numeric-add-fixed credit (cadr result)))))
+            (begin
+              (set! sale (gnc-numeric-add-fixed sale (cadddr result)))
+              (set! tax (gnc-numeric-add-fixed tax (car (cddddr result))))
+              (if (gnc-numeric-negative-p (cadr result))
+                (set! debit (gnc-numeric-add-fixed debit (cadr result)))
+                (set! credit (gnc-numeric-add-fixed credit (cadr result))))))
           (set! total (gnc-numeric-add-fixed total (cadr result)))
           (set! odd-row? (caddr result))
           ))))
@@ -423,7 +455,7 @@
       (add-balance-row table used-columns (car txns) odd-row? printed? start-date total)
         ))
 
-    (if (or (credit-col used-columns) (debit-col used-columns))
+    (if (or (sale-col used-columns) (tax-col used-columns) (credit-col used-columns) (debit-col used-columns))
     (gnc:html-table-append-row/markup! 
      table
      "grand-total"
@@ -448,6 +480,14 @@
           (gnc:make-html-table-cell/size/markup
           1 1 "total-number-cell"
           (gnc:make-gnc-monetary currency credit))))
+      (if (tax-col used-columns) (addto! row-contents
+          (gnc:make-html-table-cell/size/markup
+          1 1 "total-number-cell"
+          (gnc:make-gnc-monetary currency tax))))
+      (if (sale-col used-columns) (addto! row-contents
+          (gnc:make-html-table-cell/size/markup
+          1 1 "total-number-cell"
+          (gnc:make-gnc-monetary currency sale))))
       (if (memo-col used-columns) (set! pre-span (+ pre-span 1)))
       (if (type-col used-columns) (set! pre-span (+ pre-span 1)))
       (if (num-col used-columns) (set! pre-span (+ pre-span 1)))
@@ -477,7 +517,7 @@
        "grand-total"
        (list (gnc:make-html-table-cell/size
           1 columns-used-size
-          (make-aging-table options query interval-vec reverse? date-type)))))
+          (make-aging-table options query interval-vec reverse? date-type currency)))))
 
     table))
 
@@ -538,26 +578,30 @@
     (N_ "Display Columns") desc-header
     "ha" (N_ "Display the transaction description?") #t))
 
+  (gnc:register-inv-option
+   (gnc:make-simple-boolean-option
+    (N_ "Display Columns") sale-header
+    "haa" (N_ "Display the sale amount column?") #f))
+
+  (gnc:register-inv-option
+   (gnc:make-simple-boolean-option
+    (N_ "Display Columns") tax-header
+    "hab" (N_ "Display the tax column?") #f))
+
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
     (N_ "Display Columns") credit-header
-    "haa" (N_ "Display the period credits column?") #t))
+    "hac" (N_ "Display the period credits column?") #t))
 
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
     (N_ "Display Columns") debit-header
-    "hab" (N_ "Display a period debits column?") #t))
+    "had" (N_ "Display a period debits column?") #t))
 
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
     (N_ "Display Columns") amount-header
     "hb" (N_ "Display the transaction amount?") #t)) 
-
-  (gnc:register-inv-option
-   (gnc:make-string-option
-    gnc:pagename-general (N_ "Today Date Format")
-    "p" (N_ "The format for the date->string conversion for today's date.")
-    (gnc-default-strftime-date-format)))
   
   (gnc:register-inv-option 
    (gnc:make-multichoice-option 
@@ -620,7 +664,7 @@
      guid QOF-QUERY-OR)
 
     (xaccQueryAddSingleAccountMatch q account QOF-QUERY-AND)
-    (xaccQueryAddDateMatchTS q #f end-date #t end-date QOF-QUERY-AND)
+    (xaccQueryAddDateMatchTT q #f end-date #t end-date QOF-QUERY-AND)
     (qof-query-set-book q (gnc-get-current-book))
     q))
 
@@ -634,11 +678,11 @@
     (gnc:html-table-append-row!
      table
      (list
-      (string-expand (gnc:owner-get-name-and-address-dep owner) #\newline "<br>")))
+      (string-expand (gnc:owner-get-name-and-address-dep owner) #\newline "<br/>")))
     (gnc:html-table-append-row!
      table
-     (list "<br>"))
-    (set-last-row-style!
+     (list "<br/>"))
+    (gnc:html-table-set-last-row-style!
      table "td"
      'attribute (list "valign" "top"))
     table))
@@ -648,7 +692,7 @@
    table
    (list
     (string-append label ":&nbsp;")
-    (string-expand (gnc-print-date date) #\space "&nbsp;"))))
+    (string-expand (qof-print-date date) #\space "&nbsp;"))))
 
 (define (make-date-table)
   (let ((table (gnc:make-html-table)))
@@ -656,20 +700,15 @@
      table "table"
      'attribute (list "border" 0)
      'attribute (list "cellpadding" 0))
-    (set-last-row-style!
+    (gnc:html-table-set-last-row-style!
      table "td"
      'attribute (list "valign" "top"))
     table))
 
 (define (make-myname-table book date-format)
   (let* ((table (gnc:make-html-table))
-     (slots (qof-book-get-slots book))
-     (name (kvp-frame-get-slot-path-gslist
-        slots (append gnc:*kvp-option-path*
-                  (list gnc:*business-label* gnc:*company-name*))))
-     (addy (kvp-frame-get-slot-path-gslist
-        slots (append gnc:*kvp-option-path*
-                  (list gnc:*business-label* gnc:*company-addy*)))))
+     (name (gnc:company-info book gnc:*company-name*))
+     (addy (gnc:company-info book gnc:*company-addy*)))
 
     (gnc:html-table-set-style!
      table "table"
@@ -682,11 +721,11 @@
     (gnc:html-table-append-row! table (list (if name name "")))
     (gnc:html-table-append-row! table (list (string-expand
                          (if addy addy "")
-                         #\newline "<br>")))
+                         #\newline "<br/>")))
     (gnc:html-table-append-row! table (list
                        (strftime
                     date-format
-                    (localtime (car (gnc:get-today))))))
+                    (gnc-localtime (gnc:get-today)))))
     table))
 
 (define (make-break! document)
@@ -705,19 +744,19 @@
      (orders '())
      (query (qof-query-create-for-splits))
      (account (opt-val owner-page acct-string))
-     (start-date (gnc:timepair-start-day-time 
+     (start-date (gnc:time64-start-day-time 
       (gnc:date-option-absolute-time
        (opt-val gnc:pagename-general optname-from-date))))
-     (end-date (gnc:timepair-end-day-time 
+     (end-date (gnc:time64-end-day-time 
                (gnc:date-option-absolute-time
                (opt-val gnc:pagename-general optname-to-date))))
-     (book (gnc-get-current-book)) ;XXX Grab this from elsewhere
+     (book (gnc-account-get-book account))
+     (date-format (if (not (null? book)) (gnc:options-fancy-date book)))
      (type (opt-val "__reg" "owner-type"))
      (owner-descr (owner-string type))
-     (date-type (opt-val gnc:pagename-general optname-date-driver)) 
+     (date-type (opt-val gnc:pagename-general optname-date-driver))
      (owner (opt-val owner-page owner-descr))
      (report-title (string-append (doctype-str type) " " (_ "Report"))))
-
     (if (not (gncOwnerIsValid owner))
      (gnc:html-document-add-object!
       document
@@ -757,7 +796,7 @@
 
         (gnc:html-document-add-object!
          document
-         (make-myname-table book (opt-val gnc:pagename-general (N_ "Today Date Format"))))
+         (make-myname-table book date-format))
 
         (gnc:html-document-add-object!
          document
@@ -771,9 +810,9 @@
           (string-append
            (_ "Date Range")
            ": "
-           (gnc-print-date start-date)
+           (qof-print-date start-date)
            " - "
-           (gnc-print-date end-date))))
+           (qof-print-date end-date))))
 
         (make-break! document)
 
