@@ -3,6 +3,8 @@
 ;; for simple style elements. 
 ;; Copyright 2000 Bill Gribble <grib@gnumatic.com>
 ;; 
+;; * 2004.06.18: David Montenegro, added gnc:html-table-get-cell
+;; 
 ;; This program is free software; you can redistribute it and/or    
 ;; modify it under the terms of the GNU General Public License as   
 ;; published by the Free Software Foundation; either version 2 of   
@@ -17,10 +19,20 @@
 ;; along with this program; if not, contact:
 ;;
 ;; Free Software Foundation           Voice:  +1-617-542-5942
-;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
-;; Boston, MA  02111-1307,  USA       gnu@gnu.org
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; 
+;; NB: In this code, "markup" and "/markup" *do not* refer to
+;; style information.  Rather, they let you override the tag
+;; associated with an html-table row or cell.  Style
+;; information is stored in addition to this "markup" (in
+;; an entirely different record field).
+;; 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (define <html-table>
   (make-record-type "<html-table>"
                     '(col-headers
@@ -125,11 +137,16 @@
   (let* ((retval '())
          (push (lambda (l) (set! retval (cons l retval))))
          (style (gnc:html-table-cell-style cell)))
+    
+;    ;; why dont colspans export??!
+;    (gnc:html-table-cell-set-style! cell "td"
+;	'attribute (list "colspan"
+;	    (or (gnc:html-table-cell-colspan cell) 1)))
     (gnc:html-document-push-style doc style)
     (push (gnc:html-document-markup-start 
-           doc (gnc:html-table-cell-tag cell)
-           (sprintf #f "rowspan=%a" (gnc:html-table-cell-rowspan cell))
-           (sprintf #f "colspan=%a" (gnc:html-table-cell-colspan cell))))
+           doc (gnc:html-table-cell-tag cell)  #t
+           (format #f "rowspan=\"~a\"" (gnc:html-table-cell-rowspan cell))
+           (format #f "colspan=\"~a\"" (gnc:html-table-cell-colspan cell))))
     (for-each 
      (lambda (child) 
        (push (gnc:html-object-render child doc)))
@@ -309,14 +326,7 @@
   (record-modifier <html-table> 'num-rows))
 
 (define (gnc:html-table-num-columns table)
-  (let ((max 0))
-    (for-each 
-     (lambda (row)
-       (let ((l (length row))) 
-         (if (> l max)
-             (set! max l))))
-     (gnc:html-table-data table))
-    max))
+  (apply max (map length (gnc:html-table-data table))))
 
 (define (gnc:html-table-append-row/markup! table markup newrow)
   (let ((rownum (gnc:html-table-append-row! table newrow)))
@@ -329,14 +339,12 @@
     
 
 (define (gnc:html-table-append-row! table newrow)
-  (let* ((dd (gnc:html-table-data table))
-	 (current-num-rows (gnc:html-table-num-rows table))
-	 (new-num-rows (+ current-num-rows 1)))
-    (set! dd (cons newrow dd))
-    (gnc:html-table-set-num-rows-internal! 
-     table 
-     new-num-rows)
-    (gnc:html-table-set-data! table dd)
+  (let* ((current-num-rows (gnc:html-table-num-rows table))
+	 (new-num-rows (1+ current-num-rows)))
+    (gnc:html-table-set-num-rows-internal! table new-num-rows)
+    (gnc:html-table-set-data! table
+                              (cons (if (list? newrow) newrow (list newrow))
+                                    (gnc:html-table-data table)))
     new-num-rows))
 
 (define (gnc:html-table-remove-last-row! table)
@@ -384,27 +392,90 @@
     
     new-num-rows))
 
+;; list-set! is 0-based...
+;;   (let ((a '(0 1 2))) (list-set! a 1 "x") a)
+;;    => (0 "x" 2)
+(define (gnc:html-table-get-cell table row col)
+  (let* ((row (gnc:html-table-get-row table row)))
+    (and row (list-ref-safe row col)))
+  )
+
+(define (gnc:html-table-get-row table row)
+  (let* ((dd (gnc:html-table-data table))
+	 (len (and dd (length dd)))
+	 )
+    (and len
+	 (list-ref-safe dd (- (- len 1) row))
+	 )
+    ))
+
+;; if the 4th arg is a cell, overwrite the existing cell,
+;; otherwise, append all remaining objects to the existing cell
 (define (gnc:html-table-set-cell! table row col . objects)
   (let ((rowdata #f)
-	(row-loc #f)
-        (l (length (gnc:html-table-data table))))
+        (row-loc #f)
+        (l (length (gnc:html-table-data table)))
+        (objs (length objects))
+        )
     ;; ensure the row-data is there 
     (if (>= row l)
-	(begin
+        (begin
           (let loop ((i l))
-	    (gnc:html-table-append-row! table (list))
+            (gnc:html-table-append-row! table (list))
             (if (< i row)
                 (loop (+ i 1))))
           (set! l (gnc:html-table-num-rows table))
-	  (set! row-loc (- (- l 1) row))
+          (set! row-loc (- (- l 1) row))
           (set! rowdata (list)))
-	(begin
-	  (set! row-loc (- (- l 1) row))
-	  (set! rowdata (list-ref (gnc:html-table-data table) row-loc))))
+        (begin
+          (set! row-loc (- (- l 1) row))
+          (set! rowdata (list-ref (gnc:html-table-data table) row-loc))))
     
     ;; make a table-cell and set the data 
-    (let ((tc (gnc:make-html-table-cell)))
-      (apply gnc:html-table-cell-append-objects! tc objects)
+    (let* ((tc (gnc:make-html-table-cell))
+           (first (car objects)))
+      (if (and (equal? objs 1) (gnc:html-table-cell? first))
+          (set! tc first)
+          (apply gnc:html-table-cell-append-objects! tc objects)
+          )
+      (set! rowdata (list-set-safe! rowdata col tc))
+      
+      ;; add the row-data back to the table 
+      (gnc:html-table-set-data! 
+       table (list-set-safe! 
+              (gnc:html-table-data table) 
+              row-loc rowdata)))))
+
+;; if the 4th arg is a cell, overwrite the existing cell,
+;; otherwise, append all remaining objects to the existing cell
+(define (gnc:html-table-set-cell/tag! table row col tag . objects)
+  (let ((rowdata #f)
+        (row-loc #f)
+        (l (length (gnc:html-table-data table)))
+        (num-objs (length objects))
+        )
+    ;; ensure the row-data is there 
+    (if (>= row l)
+        (begin
+          (let loop ((i l))
+            (gnc:html-table-append-row! table (list))
+            (if (< i row)
+                (loop (+ i 1))))
+          (set! l (gnc:html-table-num-rows table))
+          (set! row-loc (- (- l 1) row))
+          (set! rowdata (list)))
+        (begin
+          (set! row-loc (- (- l 1) row))
+          (set! rowdata (list-ref (gnc:html-table-data table) row-loc))))
+    
+    ;; make a table-cell and set the data 
+    (let* ((tc (gnc:make-html-table-cell))
+           (first (car objects)))
+      (if (and (equal? num-objs 1) (gnc:html-table-cell? first))
+          (set! tc first)
+          (apply gnc:html-table-cell-append-objects! tc objects)
+          )
+      (gnc:html-table-cell-set-tag! tc tag)
       (set! rowdata (list-set-safe! rowdata col tc))
       
       ;; add the row-data back to the table 
@@ -440,7 +511,8 @@
              (rest-new (cdr newcol))
              (rest-existing (cdr existing-data))
              (rest-result (append-to-element rest-new rest-existing 
-                                             (- length-to-append 1))))
+                                             (- length-to-append 1)
+                                             width-to-make)))
           (cons (cons (widen-and-append 
                        current-existing 
                        current-new 
@@ -484,7 +556,8 @@
   ;; returns a pair, the car of which is the prepending of newcol
   ;; and existing-data, and the cdr is the remaining elements of newcol
   (define (prepend-to-element newcol existing-data length-to-append)
-    (if (= length-to-append 0) ('() . newcol)
+    (if (= length-to-append 0)
+        (cons '() newcol)
         (let* 
             ((current-new (car newcol))
              (current-existing (car existing-data))
@@ -518,6 +591,17 @@
            remaining-elements)
           #f))))
 
+;; 
+;; It would be nice to have table row/col/cell accessor functions in here.
+;; It would also be nice to have table juxtaposition functions, too.
+;; i.e., (gnc:html-table-nth-row table n)
+;;  [ CAS: how is that different from gnc:html-table-get-row ? ]
+
+;;       (gnc:html-table-append-table-horizontal table add-table)
+;; (An old merge-table used to exist inside balance-sheet.scm/GnuCash 1.8.9.)
+;; Feel free to contribute! :-)
+;; 
+
 (define (gnc:html-table-render table doc)
   (let* ((retval '())
          (push (lambda (l) (set! retval (cons l retval)))))
@@ -527,13 +611,13 @@
      (gnc:html-table-style table) (gnc:html-document-style-stack doc))
     
     (gnc:html-document-push-style doc (gnc:html-table-style table))
-    (push (gnc:html-document-markup-start doc "table"))
+    (push (gnc:html-document-markup-start doc "table" #t))
     
     ;; render the caption 
     (let ((c (gnc:html-table-caption table)))
       (if c
           (begin 
-            (push (gnc:html-document-markup-start doc "caption"))
+            (push (gnc:html-document-markup-start doc "caption" #t))
             (push (gnc:html-object-render c doc))
             (push (gnc:html-document-markup-end doc "caption")))))
     
@@ -547,7 +631,6 @@
           (begin 
             (gnc:html-document-push-style 
              doc (gnc:html-table-col-headers-style table))
-            (push (gnc:html-document-markup-start doc "tr"))
             
             ;; compile the column styles just in case there's
             ;; something interesting in the table header cells.
@@ -560,12 +643,14 @@
              #f (gnc:html-table-col-styles table))
             
             ;; render the headers 
+            (push (gnc:html-document-markup-start doc "thead" #t))
+            (push (gnc:html-document-markup-start doc "tr" #t))
             (for-each 
              (lambda (hdr) 
                (gnc:html-document-push-style 
                 doc (gnc:html-table-col-style table colnum))
                (if (not (gnc:html-table-cell? hdr))
-                   (push (gnc:html-document-markup-start doc "th")))
+                   (push (gnc:html-document-markup-start doc "th" #t)))
                (push (gnc:html-object-render hdr doc))
                (if (not (gnc:html-table-cell? hdr))
                    (push (gnc:html-document-markup-end doc "th")))
@@ -575,6 +660,9 @@
                    (set! colnum (+ (gnc:html-table-cell-colspan hdr)
                                    colnum))))
              ch)
+            (push (gnc:html-document-markup-end doc "tr"))
+            (push (gnc:html-document-markup-end doc "thead"))
+
             ;; pop the col header style 
             (gnc:html-document-pop-style doc))))
     
@@ -589,6 +677,7 @@
        #f)
      #f (gnc:html-table-col-styles table))
     
+    (push (gnc:html-document-markup-start doc "tbody" #t))
     ;; now iterate over the rows 
     (let ((rownum 0) (colnum 0))
       (for-each 
@@ -604,7 +693,7 @@
            ;; push the style for this row and write the start tag, then 
            ;; pop it again.
            (if rowstyle (gnc:html-document-push-style doc rowstyle))
-           (push (gnc:html-document-markup-start doc rowmarkup))
+           (push (gnc:html-document-markup-start doc rowmarkup #t))
            (if rowstyle (gnc:html-document-pop-style doc))
            
            ;; write the column data, pushing the right column style 
@@ -619,7 +708,7 @@
                 
                 ;; render the cell contents 
                 (if (not (gnc:html-table-cell? datum))
-                    (push (gnc:html-document-markup-start doc "td")))
+                    (push (gnc:html-document-markup-start doc "td" #t)))
                 (push (gnc:html-object-render datum doc))
                 (if (not (gnc:html-table-cell? datum))
                     (push (gnc:html-document-markup-end doc "td")))
@@ -638,8 +727,13 @@
            (set! colnum 0)
            (set! rownum (+ 1 rownum))))
        (reverse (gnc:html-table-data table))))
+    (push (gnc:html-document-markup-end doc "tbody"))
     
     ;; write the table end tag and pop the table style
     (push (gnc:html-document-markup-end doc "table"))
     (gnc:html-document-pop-style doc)
     retval))
+
+(define (gnc:html-table-set-last-row-style! table tag . rest)
+  (apply gnc:html-table-set-row-style!
+         (cons* table (1- (gnc:html-table-num-rows table)) tag rest)))
