@@ -22,12 +22,12 @@
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(use-modules (gnucash utilities))
+
 ;; returns a list with n #f (empty cell) values 
 (define (gnc:html-make-empty-cell) #f)
 (define (gnc:html-make-empty-cells n)
-  (if (> n 0)
-      (cons #f (gnc:html-make-empty-cells (- n 1)))
-      (list)))
+  (make-list n #f))
 
 (define (gnc:register-guid type guid)
   (gnc-build-url URL-TYPE-REGISTER (string-append type guid) ""))
@@ -106,12 +106,20 @@
                           value)))
 
 (define (gnc:assign-colors num-colors)
-  (define base-colors '("red" "orange" "yellow" "green"
-                        "cyan" "blue" "purple" "magenta" 
-			"orchid" "khaki" "gold" "orange"
-			"red3" "orange3" "yellow3" "green3"
-                        "cyan3" "blue3" "purple3" "magenta3" 
-  			"orchid3" "khaki3" "gold3" "orange3"))
+  ;; default CSS colours
+  ;; (define base-colors '("red" "orange" "yellow" "green"
+  ;;                       "cyan" "blue" "purple" "magenta"
+  ;;                       "orchid" "khaki" "gold" "orange"
+  ;;                       "red3" "orange3" "yellow3" "green3"
+  ;;                       "cyan3" "blue3" "purple3" "magenta3"
+  ;;                       "orchid3" "khaki3" "gold3" "orange3"))
+
+  ;; new base-colors from http://clrs.cc/ and flatuicolors.com
+  (define base-colors (list "#FF4136" "#FF851B" "#FFDC00" "#2ECC40"
+                            "#0074D9" "#001f3f" "#85144b" "#7FDBFF"
+                            "#F012BE" "#3D9970" "#39CCCC" "#f39c12"
+                            "#e74c3c" "#e67e22" "#9b59b6" "#8e44ad"
+                            "#16a085" "#d35400"))
   (define (assign-colors i)
     (if (<= num-colors i)
         '()
@@ -368,7 +376,7 @@
 ;; according to their types and show a subtotal for each group.
 ;;
 ;; <bool> show-parent-balance?: Specify whether to show balances of
-;; non-leaf accounts seperately.
+;; non-leaf accounts separately.
 ;;
 ;; <bool> show-parent-total?: Whether to show a line with the label
 ;; e.g. "Total My-Assets" and the subtotal for this account and its
@@ -785,74 +793,148 @@
     table))
 
 
-;; TODO: How 'bout factoring the "Edit report options" stuff out of
-;; these 3 functions?
+(define (gnc:html-make-generic-budget-warning report-title-string)
+  (gnc:html-make-generic-simple-warning
+    report-title-string
+    (_ "No budgets exist. You must create at least one budget.")))
 
-(define (gnc:html-make-generic-options-warning
-	 report-title-string report-id)
+
+(define (gnc:html-make-generic-simple-warning report-title-string message)
   (let ((p (gnc:make-html-text)))
     (gnc:html-text-append!
      p
-     (gnc:html-markup-h2 (string-append
-			  (_ report-title-string)
-			  ":"))
+     (gnc:html-markup-h2 (string-append report-title-string ":"))
      (gnc:html-markup-h2 "")
-     (gnc:html-markup-p
-      (_ "This report requires you to specify certain report options.")))
-    (if report-id
-	(gnc:html-text-append!
-	 p
-	 (gnc:html-markup-p
-	  (gnc:html-markup-anchor
-	   (gnc-build-url URL-TYPE-OPTIONS
-			       (string-append "report-id="
-					      (sprintf #f "%a" report-id))
-			       "")
-	   (_ "Edit report options")))))
+     (gnc:html-markup-p message))
     p))
 
 
+(define (gnc:html-make-options-link report-id)
+   (if report-id
+    (gnc:html-markup-p
+     (gnc:html-markup-anchor
+      (gnc-build-url URL-TYPE-OPTIONS
+                     (format #f "report-id=~a" report-id)
+                     "")
+      (_ "Edit report options")))))
+
+(define* (gnc:html-render-options-changed options #:optional plaintext?)
+  ;; options -> html-object or string, depending on plaintext?.  This
+  ;; summarises options that were changed by the user. Set plaintext?
+  ;; to #t for unit-tests only.
+  (define (disp d)
+    ;; option-value -> string.  The option is passed to various
+    ;; scm->string converters; ultimately a generic stringify
+    ;; function handles symbol/string/other types.
+    (define (try proc)
+      ;; Try proc with d as a parameter, catching 'wrong-type-arg
+      ;; exceptions to return #f to the or evaluator.
+      (catch 'wrong-type-arg
+        (lambda () (proc d))
+        (const #f)))
+    (or (and (boolean? d) (if d (_ "Enabled") (_ "Disabled")))
+        (and (null? d) "null")
+        (and (list? d) (string-join (map disp d) ", "))
+        (and (pair? d) (format #f "~a . ~a"
+                               (car d)
+                               (if (eq? (car d) 'absolute)
+                                   (qof-print-date (cdr d))
+                                   (disp (cdr d)))))
+        (try gnc-commodity-get-mnemonic)
+        (try xaccAccountGetName)
+        (try gnc-budget-get-name)
+        (format #f "~a" d)))
+  (let ((render-list '()))
+    (define (add-option-if-changed option)
+      (let* ((section (gnc:option-section option))
+             (name (gnc:option-name option))
+             (default-value (gnc:option-default-value option))
+             (value (gnc:option-value option))
+             (retval (cons (format #f "~a / ~a" section name)
+                           (disp value))))
+        (if (not (or (equal? default-value value)
+                     (char=? (string-ref section 0) #\_)))
+            (addto! render-list retval))))
+    (gnc:options-for-each add-option-if-changed options)
+    (if plaintext?
+        (string-append
+         (string-join
+          (map (lambda (item)
+                 (format #f "~a: ~a\n" (car item) (cdr item)))
+               render-list)
+          "")
+         "\n")
+        (apply
+         gnc:make-html-text
+         (apply
+          append
+          (map
+           (lambda (item)
+             (list
+              (gnc:html-markup-b (car item))
+              ": "
+              (cdr item)
+              (gnc:html-markup-br)))
+           render-list))))))
+
+(define (gnc:html-make-generic-warning
+         report-title-string report-id
+         warning-title-string warning-string)
+  (let ((p (gnc:make-html-text)))
+   (gnc:html-text-append!
+    p
+    (gnc:html-markup-h2 (string-append (_ report-title-string) ":"))
+    (gnc:html-markup-h2 warning-title-string)
+    (gnc:html-markup-p warning-string)
+    (gnc:html-make-options-link report-id))
+   p))
+
+(define (gnc:html-make-generic-options-warning
+         report-title-string report-id)
+  (gnc:html-make-generic-warning
+    report-title-string
+    report-id
+    ""
+    (_ "This report requires you to specify certain report options.")))
+
 (define (gnc:html-make-no-account-warning
-	 report-title-string report-id)
-  (let ((p (gnc:make-html-text)))
-    (gnc:html-text-append! 
-     p 
-     (gnc:html-markup-h2 (string-append 
-			  (_ report-title-string)
-			  ":"))
-     (gnc:html-markup-h2 (_ "No accounts selected"))
-     (gnc:html-markup-p
-      (_ "This report requires accounts to be selected.")))
-    (if report-id
-	(gnc:html-text-append! 
-	 p 
-	 (gnc:html-markup-p
-	  (gnc:html-markup-anchor
-	   (gnc-build-url URL-TYPE-OPTIONS
-			       (string-append "report-id="
-					      (sprintf #f "%a" report-id))
-			       "")
-	   (_ "Edit report options")))))
-    p))
+         report-title-string report-id)
+  (gnc:html-make-generic-warning
+    report-title-string
+    report-id
+    (_ "No accounts selected")
+    (_ "This report requires accounts to be selected in the report options.")))
+
+(define (gnc:html-make-empty-data-warning
+         report-title-string report-id)
+  (gnc:html-make-generic-warning
+    report-title-string
+    report-id
+    (_ "No data")
+    (_ "The selected accounts contain no data/transactions (or only zeroes) for the selected time period")))
+
+(define (gnc:html-js-include file)
+  (format #f
+          "<script language=\"javascript\" type=\"text/javascript\" src=\"file:///~a\"></script>\n"
+          (gnc-path-find-localized-html-file file)))
+
+(define (gnc:html-css-include file)
+  (format #f
+          "<link rel=\"stylesheet\" type=\"text/css\" href=\"file:///~a\" />\n"
+          (gnc-path-find-localized-html-file file)))
+
+;; function to sanitize strings prior to sending to html
+(define (gnc:html-string-sanitize str)
+  (with-output-to-string
+    (lambda ()
+      (string-for-each
+       (lambda (c)
+         (display
+          (case c
+            ((#\&) "&amp;")
+            ((#\<) "&lt;")
+            ((#\>) "&gt;")
+            (else c))))
+       str))))
+
 
-(define (gnc:html-make-empty-data-warning 
-	 report-title-string report-id)
-  (let ((p (gnc:make-html-text)))
-    (gnc:html-text-append! 
-     p 
-     (gnc:html-markup-h2 
-      (string-append report-title-string ":"))
-     (gnc:html-markup-h2 (_ "No data"))
-     (gnc:html-markup-p
-      (_ "The selected accounts contain no data/transactions (or only zeroes) for the selected time period")))
-    (if report-id
-	(gnc:html-text-append! 
-	 p 
-	 (gnc:html-markup-p
-	  (gnc:html-markup-anchor
-	   (gnc-build-url URL-TYPE-OPTIONS
-			       (string-append "report-id="
-					      (sprintf #f "%a" report-id))
-			       "")
-	   (_ "Edit report options")))))
-    p))
