@@ -5,9 +5,28 @@
 ;;;
 ;;;  Copyright 2000-2001 Bill Gribble <grib@billgribble.com>
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; This program is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2 of
+;; the License, or (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 
 (use-modules (srfi srfi-13))
-
+(use-modules (gnucash import-export string))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;  qif-import:find-or-make-acct
@@ -45,12 +64,12 @@
       (if (not (null? (gnc-account-lookup-by-full-name old-root long-name)))
           (let loop ((count 2))
             (let* ((test-name
-                    (string-append long-name (sprintf #f " %a" count)))
+                    (string-append long-name (format #f " ~a" count)))
                    (test-acct
                     (gnc-account-lookup-by-full-name old-root test-name)))
               (if (and (not (null? test-acct)) (not (compatible? test-acct)))
                   (loop (+ 1 count))
-                  (string-append short-name (sprintf #f " %a" count)))))
+                  (string-append short-name (format #f " ~a" count)))))
           short-name))
 
     ;; If a GnuCash account already exists in the old root with the same
@@ -133,6 +152,8 @@
                  new-acct (xaccAccountGetCommodity same-gnc-account))
                 (xaccAccountSetNotes
                  new-acct (xaccAccountGetNotes same-gnc-account))
+                (xaccAccountSetColor
+                 new-acct (xaccAccountGetColor same-gnc-account))
                 (xaccAccountSetCode
                  new-acct (xaccAccountGetCode same-gnc-account))))
 
@@ -212,7 +233,9 @@
 (define (qif-import:qif-to-gnc qif-files-list
                                qif-acct-map qif-cat-map
                                qif-memo-map stock-map
-                               default-currency-name progress-dialog)
+                               default-currency-name
+                               transaction-status-pref
+                               progress-dialog)
 
   ;; This procedure does all the work. We'll define it, then call it safely.
   (define (private-convert)
@@ -232,7 +255,11 @@
                                              (length (qif-file:xtns b))))))
            (work-to-do 0)
            (work-done 0))
-
+      ;; Log any errors
+      (define (errorproc message)
+        (if (string? message)
+              (qif-import:log progress-dialog "qif-import:qif-to-gnc"
+                            message)))
       ;; This procedure handles progress reporting, pause, and cancel.
       (define (update-progress)
         (set! work-done (+ 1 work-done))
@@ -356,7 +383,7 @@
             (update-progress)
 
             (if (not (qif-xtn:mark xtn))
-                (qif-import:mark-matching-xtns xtn rest))
+                (qif-import:mark-matching-xtns xtn rest errorproc))
             (if (not (null? (cdr rest)))
                 (xloop (car rest) (cdr rest)))))
 
@@ -379,8 +406,9 @@
                                 (gnc-get-current-book))))
                   (xaccTransBeginEdit gnc-xtn)
 
-                  ;; FIXME. This is probably wrong
-                  (xaccTransSetCurrency gnc-xtn (gnc-default-currency))
+                  ;; All accounts & splits are required to be in the
+                  ;; user-specified currency. Use it for the txn too.
+                  (xaccTransSetCurrency gnc-xtn default-currency)
 
                   ;; Build the transaction.
                   (qif-import:qif-xtn-to-gnc-xtn xtn qif-file gnc-xtn
@@ -388,6 +416,7 @@
                                                  qif-acct-map
                                                  qif-cat-map
                                                  qif-memo-map
+                                                 transaction-status-pref
                                                  progress-dialog)
 
                   ;; rebalance and commit everything
@@ -406,7 +435,7 @@
     (lambda ()
       (catch 'cancel
              (lambda ()
-               (catch 'bad-date private-convert (lambda (key . args) key)))
+               (catch #t private-convert (lambda (key . args) key)))
              (lambda (key . args) #t)))))
 
 
@@ -419,6 +448,7 @@
 (define (qif-import:qif-xtn-to-gnc-xtn qif-xtn qif-file gnc-xtn
                                        gnc-acct-hash
                                        qif-acct-map qif-cat-map qif-memo-map
+                                       transaction-status-pref
                                        progress-dialog)
   (let ((splits (qif-xtn:splits qif-xtn))
         (gnc-near-split (xaccMallocSplit (gnc-get-current-book)))
@@ -454,15 +484,7 @@
                "Missing transaction date."
                #f
                #f))
-      ((< (list-ref qif-date 2) 1970)
-        (qif-import:log progress-dialog
-                        "qif-import:qif-xtn-to-gnc-xtn"
-                        (_ "Dates earlier than 1970 are not supported."))
-        (throw 'bad-date
-               "qif-import:qif-xtn-to-gnc-xtn"
-               "Invalid transaction year (~A)."
-               (list (list-ref qif-date 2))
-               #f))
+ 
       (else
         (apply xaccTransSetDate gnc-xtn (qif-xtn:date qif-xtn))))
 
@@ -470,7 +492,9 @@
     (if qif-payee
         (xaccTransSetDescription gnc-xtn qif-payee))
     (if qif-number
-        (xaccTransSetNum gnc-xtn qif-number))
+        ;; Use function that will set either tran-num or split-action per
+        ;; book option.
+        (gnc-set-num-action gnc-xtn gnc-near-split qif-number #f))
 
     ;; Look for the transaction memo (QIF "M" line). When a default split
     ;; exists, the memo can be found there. Otherwise, it will be in the
@@ -486,10 +510,15 @@
 	      ;; the debit/credit lines. See bug 495219 for more information.
 	      (xaccTransSetNotes gnc-xtn qif-memo)))
 
+    ;; Look for the transaction status (QIF "C" line). When it exists, apply
+    ;; the cleared (c) or reconciled (y) status to the split. Otherwise, apply
+    ;; user preference.
     (if (eq? qif-cleared 'cleared)
-        (xaccSplitSetReconcile gnc-near-split #\c))
-    (if (eq? qif-cleared 'reconciled)
-        (xaccSplitSetReconcile gnc-near-split #\y))
+        (xaccSplitSetReconcile gnc-near-split #\c)
+        (if (eq? qif-cleared 'reconciled)
+            (xaccSplitSetReconcile gnc-near-split #\y)
+            ;; Apply user preference by default.
+            (xaccSplitSetReconcile gnc-near-split transaction-status-pref)))
 
     (if (not qif-security)
         (begin
@@ -785,7 +814,7 @@
 ;;  mark them so they won't be imported.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (qif-import:mark-matching-xtns xtn candidate-xtns)
+(define (qif-import:mark-matching-xtns xtn candidate-xtns errorproc)
   (let splitloop ((splits-left (qif-xtn:splits xtn)))
 
     ;; splits-left starts out as all the splits of this transaction.
@@ -797,7 +826,7 @@
                  (qif-split:category-is-account? (car splits-left)))
             (set! splits-left
                   (qif-import:mark-some-splits
-                   splits-left xtn candidate-xtns))
+                   splits-left xtn candidate-xtns errorproc))
             (set! splits-left (cdr splits-left))))
 
     (if (not (null? splits-left))
@@ -810,7 +839,7 @@
 ;; don't get imported.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (qif-import:mark-some-splits splits xtn candidate-xtns)
+(define (qif-import:mark-some-splits splits xtn candidate-xtns errorproc)
   (let* ((n- (lambda (n) (gnc-numeric-neg n)))
          (nsub (lambda (a b) (gnc-numeric-sub a b 0 GNC-DENOM-LCD)))
          (n+ (lambda (a b) (gnc-numeric-add a b 0 GNC-DENOM-LCD)))
@@ -885,17 +914,20 @@
     ;; this is the grind loop.  Go over every unmarked transaction in
     ;; the candidate-xtns list.
     (let xtn-loop ((xtns candidate-xtns))
-      (if (and (not (qif-xtn:mark (car xtns)))
-               (string=? (qif-xtn:from-acct (car xtns)) far-acct-name))
-          (begin
-            (set! how
-                  (qif-import:xtn-has-matches? (car xtns) near-acct-name
-                                               date amount group-amount))
-            (if how
-                (begin
-                  (qif-import:merge-and-mark-xtns xtn same-acct-splits
-                                                  (car xtns) how)
-                  (set! done #t)))))
+      (if (not (and far-acct-name near-acct-name))
+          (if errorproc
+              (errorproc "Transaction with no or only one associated account."))
+          (if (and (not (qif-xtn:mark (car xtns))))
+              (string=? (qif-xtn:from-acct (car xtns)) far-acct-name)
+              (begin
+                (set! how
+                      (qif-import:xtn-has-matches? (car xtns) near-acct-name
+                                                   date amount group-amount))
+                (if how
+                    (begin
+                      (qif-import:merge-and-mark-xtns xtn same-acct-splits
+                                                      (car xtns) how)
+                      (set! done #t))))))
       ;; iterate with the next transaction
       (if (and (not done)
                (not (null? (cdr xtns))))
