@@ -18,7 +18,6 @@
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 
 (use-modules (srfi srfi-13))
-(use-modules (gnucash printf))
 
 (define (list-ref-safe list elt)
   (if (> (length list) elt)
@@ -73,7 +72,7 @@
 ;; one of the type identifiers in typelist.
 (define (gnc:filter-accountlist-type typelist accounts)
   (filter (lambda (a) 
-	    (member (xaccAccountGetType a) typelist))
+	    (and (not (null? a)) (member (xaccAccountGetType a) typelist)))
 	  accounts))
 
 ;; Decompose a given list of accounts 'accounts' into an alist
@@ -143,9 +142,6 @@
   (let ((root (gnc-get-current-root-account)))
     (gnc-account-get-tree-depth root)))
 
-(define (gnc:split-get-corr-account-full-name split)
-  (xaccSplitGetCorrAccountFullName split))
-
 
 ;; Get all children of this list of accounts.
 (define (gnc:acccounts-get-all-subaccounts accountlist)
@@ -269,24 +265,24 @@
 
 
 ;; Same as above but with gnc:numeric
-(define (gnc:make-numeric-collector)
+(define (gnc:make-number-collector)
   (let ;;; values
-      ((value (gnc-numeric-zero)))
+      ((value 0))
     (lambda (action amount)  ;;; Dispatch function
       (case action
-	((add) (if (gnc:gnc-numeric? amount) 
-                   (set! value (gnc-numeric-add amount value
-                                                GNC-DENOM-AUTO GNC-DENOM-LCD))
-                   (gnc:warn 
-                    "gnc:numeric-collector called with wrong argument: "
+	((add) (if (number? amount)
+                     (set! value (gnc-numeric-add amount value
+                                                  GNC-DENOM-AUTO GNC-DENOM-LCD))
+                   (gnc:warn
+                    "gnc:Number-collector called with wrong argument: "
                     amount)))
 	((total) value)
-	(else (gnc:warn "bad gnc:numeric-collector action: " action))))))
+	(else (gnc:warn "bad gnc:number-collector action: " action))))))
 
 ;; Replace all 'action function calls by the normal functions below.
-(define (gnc:numeric-collector-add collector amount)
+(define (gnc:number-collector-add collector amount)
   (collector 'add amount))
-(define (gnc:numeric-collector-total collector)
+(define (gnc:number-collector-total collector)
   (collector 'total #f))
 
 ;; A commodity collector. This is intended to handle multiple
@@ -335,15 +331,18 @@
     ;; If no pair with this commodity exists, we will create one.
     (define (add-commodity-value commodity value)
       ;; lookup the corresponding pair
-      (let ((pair (assoc commodity commoditylist)))
+      (let ((pair (assoc commodity commoditylist))
+            (rvalue (gnc-numeric-convert
+                     value
+                     (gnc-commodity-get-fraction commodity) GNC-RND-ROUND)))
 	(if (not pair)
 	    (begin
-	      ;; create a new pair, using the gnc:numeric-collector
-	      (set! pair (list commodity (gnc:make-numeric-collector)))
+	      ;; create a new pair, using the gnc:number-collector
+	      (set! pair (list commodity (gnc:make-number-collector)))
 	      ;; and add it to the alist
 	      (set! commoditylist (cons pair commoditylist))))
 	;; add the value
-	(gnc:numeric-collector-add (cadr pair) value)))
+	(gnc:number-collector-add (cadr pair) rvalue)))
     
     ;; helper function to walk an association list, adding each
     ;; (commodity -> collector) pair to our list at the appropriate 
@@ -352,7 +351,7 @@
       (cond ((null? clist) '())
 	    (else (add-commodity-value 
 		   (caar clist) 
-		   (gnc:numeric-collector-total (cadar clist)))
+		   (gnc:number-collector-total (cadar clist)))
 		  (add-commodity-clist (cdr clist)))))
 
     (define (minus-commodity-clist clist)
@@ -360,7 +359,7 @@
 	    (else (add-commodity-value 
 		   (caar clist) 
 		   (gnc-numeric-neg
-		    (gnc:numeric-collector-total (cadar clist))))
+		    (gnc:number-collector-total (cadar clist))))
 		  (minus-commodity-clist (cdr clist)))))
 
     ;; helper function walk the association list doing a callback on
@@ -368,7 +367,7 @@
     (define (process-commodity-list fn clist)
       (map 
        (lambda (pair) (fn (car pair) 
-			  (gnc:numeric-collector-total (cadr pair))))
+			  (gnc:number-collector-total (cadr pair))))
        clist))
 
     ;; helper function which is given a commodity and returns, if
@@ -381,8 +380,8 @@
 		  (gnc-numeric-zero)
 		  (if sign?
 		      (gnc-numeric-neg
-		       (gnc:numeric-collector-total (cadr pair)))
-		      (gnc:numeric-collector-total (cadr pair))))
+		       (gnc:number-collector-total (cadr pair)))
+		      (gnc:number-collector-total (cadr pair))))
 	      '()))))
 
     ;; helper function which is given a commodity and returns, if
@@ -395,8 +394,8 @@
 	       (gnc-numeric-zero)
 	       (if sign?
 		   (gnc-numeric-neg
-		    (gnc:numeric-collector-total (cadr pair)))
-		   (gnc:numeric-collector-total (cadr pair)))))))
+		    (gnc:number-collector-total (cadr pair)))
+		   (gnc:number-collector-total (cadr pair)))))))
     
     ;; Dispatch function
     (lambda (action commodity amount)
@@ -489,7 +488,7 @@
 
       (qof-query-set-book query (gnc-get-current-book))
       (xaccQueryAddSingleAccountMatch query account QOF-QUERY-AND)
-      (xaccQueryAddDateMatchTS query #f date #t date QOF-QUERY-AND)
+      (xaccQueryAddDateMatchTT query #f date #t date QOF-QUERY-AND)
       (qof-query-set-sort-order query
 				(list SPLIT-TRANS TRANS-DATE-POSTED)
 				(list QUERY-DEFAULT-SORT)
@@ -529,9 +528,9 @@
     ;; Build a query to find all splits between the indicated dates.
     (qof-query-set-book query (gnc-get-current-book))
     (xaccQueryAddSingleAccountMatch query account QOF-QUERY-AND)
-    (xaccQueryAddDateMatchTS query
-                             (and start-date #t) start-date
-                             (and end-date #t) end-date
+    (xaccQueryAddDateMatchTT query
+                             (and start-date #t) (if start-date start-date 0)
+                             (and end-date #t) (if end-date end-date 0)
                              QOF-QUERY-AND)
 
     ;; Get the query results.
@@ -686,20 +685,22 @@
     (xaccTransGetVoidStatus trans)))
 
 (define (gnc:report-starting report-name)
-  (gnc-window-show-progress (sprintf #f
-				     (_ "Building '%s' report ...")
+  (gnc-window-show-progress (format #f
+				     (_ "Building '~a' report ...")
 				     (gnc:gettext report-name))
 			    0))
 
 (define (gnc:report-render-starting report-name)
-  (gnc-window-show-progress (sprintf #f
-				     (_ "Rendering '%s' report ...")
+  (gnc-window-show-progress (format #f
+				     (_ "Rendering '~a' report ...")
 				     (if (string-null? report-name)
 					 (gnc:gettext "Untitled")
 					 (gnc:gettext report-name)))
 			    0))
 
 (define (gnc:report-percent-done percent)
+  (if (> percent 100)
+      (gnc:warn "report more than 100% finished. " percent))
   (gnc-window-show-progress "" percent))
 
 (define (gnc:report-finished)
@@ -716,14 +717,13 @@
 ;; the type is an alist '((str "match me") (cased #f) (regexp #f))
 ;; If type is #f, sums all non-closing splits in the interval
 (define (gnc:account-get-trans-type-balance-interval
-	 account-list type start-date-tp end-date-tp)
+	 account-list type start-date end-date)
   (let* ((total (gnc:make-commodity-collector)))
     (map (lambda (split)
            (let* ((shares (xaccSplitGetAmount split))
                   (acct-comm (xaccAccountGetCommodity
                               (xaccSplitGetAccount split)))
-                  (txn (xaccSplitGetParent split))
-                  )
+                  (txn (xaccSplitGetParent split)))
              (if type 
                 (gnc-commodity-collector-add total acct-comm shares)
                 (if (not (xaccTransGetIsClosingTxn txn))
@@ -731,7 +731,7 @@
              )))
            )
 	 (gnc:account-get-trans-type-splits-interval
-          account-list type start-date-tp end-date-tp)
+          account-list type start-date end-date)
 	 )
     total
     )
@@ -741,7 +741,7 @@
 ;; the type is an alist '((str "match me") (cased #f) (regexp #f))
 ;; If type is #f, sums all splits in the interval (even closing splits)
 (define (gnc:account-get-trans-type-balance-interval-with-closing
-	 account-list type start-date-tp end-date-tp)
+	 account-list type start-date end-date)
   (let* ((total (gnc:make-commodity-collector)))
     (map (lambda (split)
            (let* ((shares (xaccSplitGetAmount split))
@@ -752,15 +752,83 @@
              )
            )
 	 (gnc:account-get-trans-type-splits-interval
-          account-list type start-date-tp end-date-tp)
+          account-list type start-date end-date)
 	 )
     total
     )
   )
+
+;; Filters the splits from the source to the target accounts
+;; returns a commodity collector
+;; does NOT do currency exchanges
+(define (gnc:account-get-total-flow direction target-account-list from-date to-date)
+
+  (let* (
+          (total-flow (gnc:make-commodity-collector))
+        )
+
+    ;; ------------------------------------------------------------------
+    ;; process all target accounts
+    ;; ------------------------------------------------------------------
+    (for-each
+      (lambda (target-account)
+        ;; -------------------------------------
+        ;; process all splits of current account
+        ;; -------------------------------------
+        (for-each
+          (lambda (target-account-split)
+            ;; ----------------------------------------------------
+            ;; only target account splits that are within the specified time range
+            ;; ----------------------------------------------------
+            (let* (
+                    (transaction (xaccSplitGetParent target-account-split))
+                    (transaction-date-posted (xaccTransGetDate transaction))
+                  )
+              (if (and
+                    (<= transaction-date-posted to-date)
+                    (>= transaction-date-posted from-date)
+                  )
+                ;; -------------------------------------------------------------
+                ;; get the split information
+                ;; -------------------------------------------------------------
+                (let* (
+                        (transaction-currency   (xaccTransGetCurrency transaction))
+                        (transaction-value (gnc-numeric-zero))
+                        (split-value       (xaccSplitGetAmount target-account-split))
+                      )
+                  ;; -------------------------------------------------------------
+                  ;; update the return value
+                  ;; -------------------------------------------------------------
+                  (case direction
+                    ((in)
+                      (if (gnc-numeric-positive-p split-value)
+                        (total-flow 'add transaction-currency split-value)
+                      )
+                    )
+                    ((out)
+                      (if (gnc-numeric-negative-p split-value)
+                        (total-flow 'add transaction-currency split-value)
+                      )
+                    )
+                    (else  (gnc:warn  "bad gnc:account-get-total-flow action: "  direction))
+                  )
+                )
+              )
+            )
+          )
+          (xaccAccountGetSplitList target-account)
+        )
+      )
+      target-account-list
+    )
+    total-flow ;; RETURN
+  )
+)
+
 ;; similar, but only counts transactions with non-negative shares and
 ;; *ignores* any closing entries
 (define (gnc:account-get-pos-trans-total-interval
-	 account-list type start-date-tp end-date-tp)
+	 account-list type start-date end-date)
   (let* ((str-query (qof-query-create-for-splits))
 	 (sign-query (qof-query-create-for-splits))
 	 (total-query #f)
@@ -780,16 +848,18 @@
     (gnc:query-set-match-non-voids-only! sign-query (gnc-get-current-book))
     (xaccQueryAddAccountMatch str-query account-list QOF-GUID-MATCH-ANY QOF-QUERY-AND)
     (xaccQueryAddAccountMatch sign-query account-list QOF-GUID-MATCH-ANY QOF-QUERY-AND)
-    (xaccQueryAddDateMatchTS
+    (xaccQueryAddDateMatchTT
      str-query
-     (and start-date-tp #t) start-date-tp
-     (and end-date-tp #t) end-date-tp QOF-QUERY-AND)
-    (xaccQueryAddDateMatchTS
+     (and start-date #t) (if start-date start-date 0)
+     (and end-date #t) (if end-date end-date 0)
+     QOF-QUERY-AND)
+    (xaccQueryAddDateMatchTT
      sign-query
-     (and start-date-tp #t) start-date-tp
-     (and end-date-tp #t) end-date-tp QOF-QUERY-AND)
+     (and start-date #t) (if start-date start-date 0)
+     (and end-date #t) (if end-date end-date 0)
+     QOF-QUERY-AND)
     (xaccQueryAddDescriptionMatch
-     str-query matchstr case-sens regexp QOF-QUERY-AND)
+     str-query matchstr case-sens regexp QOF-COMPARE-CONTAINS QOF-QUERY-AND)
     (set! total-query
           ;; this is a tad inefficient, but its a simple way to accomplish
           ;; description match inversion...
@@ -815,19 +885,24 @@
          splits
          )
     (qof-query-destroy total-query)
-    total
-    )
-  )
+    total))
 
 ;; Return the splits that match an account list, date range, and (optionally) type
-;; where type is defined as an alist '((str "match me") (cased #f) (regexp #f))
+;; where type is defined as an alist like:
+;; '((str "match me") (cased #f) (regexp #f) (closing #f))
+;; where str, cased, and regexp define a pattern match on transaction deseriptions 
+;; and "closing" matches transactions created by the book close command.  If "closing"
+;; is given as #t then only closing transactions will be returned, if it is #f then
+;; only non-closing transactions will be returned, and if it is omitted then both
+;; kinds of transactions will be returned.
 (define (gnc:account-get-trans-type-splits-interval
-         account-list type start-date-tp end-date-tp)
+         account-list type start-date end-date)
   (if (null? account-list)
       ;; No accounts given. Return empty list.
       '()
       ;; The normal case: There are accounts given.
   (let* ((query (qof-query-create-for-splits))
+         (query2 #f)
 	 (splits #f)
 	 (get-val (lambda (alist key)
 		    (let ((lst (assoc-ref alist key)))
@@ -835,17 +910,26 @@
 	 (matchstr (get-val type 'str))
 	 (case-sens (if (get-val type 'cased) #t #f))
 	 (regexp (if (get-val type 'regexp) #t #f))
+	 (closing (if (get-val type 'closing) #t #f))
 	 )
     (qof-query-set-book query (gnc-get-current-book))
     (gnc:query-set-match-non-voids-only! query (gnc-get-current-book))
     (xaccQueryAddAccountMatch query account-list QOF-GUID-MATCH-ANY QOF-QUERY-AND)
-    (xaccQueryAddDateMatchTS
+    (xaccQueryAddDateMatchTT
      query
-     (and start-date-tp #t) start-date-tp
-     (and end-date-tp #t) end-date-tp QOF-QUERY-AND)
-    (if type (xaccQueryAddDescriptionMatch
-              query matchstr case-sens regexp QOF-QUERY-AND))
-    
+     (and start-date #t) (if start-date start-date 0)
+     (and end-date #t) (if end-date end-date 0)
+     QOF-QUERY-AND)
+    (if (or matchstr closing) 
+         (begin
+           (set! query2 (qof-query-create-for-splits))
+           (if matchstr (xaccQueryAddDescriptionMatch
+                         query2 matchstr case-sens regexp QOF-COMPARE-CONTAINS QOF-QUERY-OR))
+           (if closing (xaccQueryAddClosingTransMatch query2 1 QOF-QUERY-OR))
+           (qof-query-merge-in-place query query2 QOF-QUERY-AND)
+           (qof-query-destroy query2)
+    ))
+
     (set! splits (qof-query-run query))
     (qof-query-destroy query)
     splits
